@@ -1,12 +1,16 @@
-import settings
-
-from GConfBackend import GConfBackend as Backend
+if (HAVE_WIN32):
+    from Win32Backend import Win32Backend as Backend
+else:
+    from GConfBackend import GConfBackend as Backend
 
 
 class _ConfigManager(Backend):
 
     """Class for managing configuration."""
 
+    UNDEF = Backend.UNDEF
+    
+
     def __init__(self):
 
         Backend.__init__(self)
