@@ -1,12 +1,5 @@
 from utils.Observable import Observable
-
-try:
-    import gconf
-except ImportError:
-    print "The GConf python bindings aren't (correctly) installed."
-    print "Install this package, exiting!"
-    import sys
-    sys.exit(1)
+import gconf
 
 
 class GConfBackend(Observable):
@@ -14,6 +7,9 @@ class GConfBackend(Observable):
     OBS_UPDATE = 0
     CONFIG_LOCATION = "/apps/gdesklets"
 
+    UNDEF = "-- undef --"
+
+
     def __init__(self):
 
         self.__client = gconf.client_get_default()
@@ -86,7 +82,7 @@ class GConfBackend(Observable):
             else:
                 raise ValueError("Invalid data type %s." % (dtype))
         else:
-            return ""
+            return self.UNDEF
 
 
 
