@@ -4,6 +4,7 @@ from utils.datatypes import *
 from utils import Unit
 from main import _
 
+import gobject
 import gtk
 
 
@@ -12,8 +13,9 @@ import gtk
 #
 class ContainerTarget(DisplayTarget):
 
+    __slots__ = ('__children', '__children_id', '__ids', '__relatives')
 
-    def __init__(self, parent):
+    def __init__(self, name, parent):
 
         # the list is necessary for preserving order
         self.__children = []
@@ -27,7 +29,7 @@ class ContainerTarget(DisplayTarget):
         # the relative-relations: id_of_relative -> [child]
         self.__relatives = {}
 
-        DisplayTarget.__init__(self, parent)
+        DisplayTarget.__init__(self, name, parent)
 
 
 
@@ -72,7 +74,10 @@ class ContainerTarget(DisplayTarget):
 
         import targetregistry
         cid = settings["id"]
-        child = targetregistry.create(childtype, self)
+        try:
+            child = targetregistry.create(childtype, self)
+        except KeyError:
+            return
         self._register_child(child, cid)
         child.get_widget().show()
 
@@ -123,9 +128,7 @@ class ContainerTarget(DisplayTarget):
         if (not children): return
         elif (len(children) > 1):
             name = "???"
-            msg = _("The %(name)s container accepts only one child.")
-            print msg % vars()
-
+            log("The %(name)s container accepts only one child." % vars())
             return children[0]
 
         else: return children[0]
@@ -133,9 +136,6 @@ class ContainerTarget(DisplayTarget):
 
 
     def get_child_by_id(self, ident): return self.__children_id.get(ident)
-
-
-
     def get_id_by_child(self, child): return self.__ids.get(child)
 
 
@@ -199,8 +199,6 @@ class ContainerTarget(DisplayTarget):
                 if (uh.get_unit() != Unit.UNIT_PERCENT):
                     by2 = max(by2, cy + ch)
 
-
-
         bx1 = min(bx1, bx2)
         by1 = min(by1, by2)
         bx2 = max(bx1, bx2)
@@ -239,10 +237,10 @@ class ContainerTarget(DisplayTarget):
         bw += bw1 + bw2
         bh += bh1 + bh2
 
-        if (x != None): return x
-        elif (y != None): return y
-        elif (width != None): return bx + bw
-        elif (height != None): return by + bh
+        if (x): return x
+        elif (y): return y
+        elif (width): return bx + bw
+        elif (height): return by + bh
 
 
 
@@ -253,7 +251,7 @@ class ContainerTarget(DisplayTarget):
 
         parent = self._get_parent()
         if (parent != self._get_display() and not self.is_standalone()):
-            gtk.timeout_add(0, parent.adjust_geometry)
+            parent.request_adjust_geometry()
 
         # tell percentual children about the new size
         for c in self._get_children():
@@ -262,8 +260,7 @@ class ContainerTarget(DisplayTarget):
             ux, uy, uw, uh = c.get_user_geometry()
             if (Unit.UNIT_PERCENT in (ux.get_unit(), uy.get_unit(),
                                       uw.get_unit(), uh.get_unit())):
-                                 #c.get_geometry_units()):
-                c.adjust_geometry()
+                c.request_adjust_geometry()
 
         self.update_observer(self.OBS_GEOMETRY)
 
@@ -282,7 +279,9 @@ class ContainerTarget(DisplayTarget):
             cx, cy, cw, ch = c.get_geometry()
             if (cx <= px <= cx + cw and cy <= py <= cy + ch):
                 c.handle_action(action, px - cx, py - cy, args)
-                c.notify_handle_action(True)
+                # don't propagate True when leaving the window
+                if (action != self.ACTION_LEAVE):
+                    c.notify_handle_action(True)
             else:
                 c.notify_handle_action(False)
         #end for
@@ -300,3 +299,4 @@ class ContainerTarget(DisplayTarget):
                 if (c.is_standalone()): continue
                 c.notify_handle_action(False)
         #end if
+
