@@ -1,16 +1,20 @@
-from config.ConfigManager import ConfigManager
+from config.DisplayConfigger import DisplayConfigger
+from config.StateSaver import DefaultStateSaver
 from DisplayTarget import DisplayTarget
 from TargetGroup import TargetGroup
 from main import UNSET_COORD
 from scripting.Script import Script
+from scripting.Scriptlet import Scriptlet
 from utils import actionparser
 from utils.Struct import Struct
 from utils.Observable import Observable
 from utils import Unit
+from utils import vfs
+import utils
 
-import NullObject
 import utils.dialog
 
+import gobject
 import gtk
 import time
 import random
@@ -18,40 +22,58 @@ import sys
 import os
 import re
 
-try:
-    import gnomevfs
-except ImportError:
-    import gnome.vfs as gnomevfs
+
+# The name of the namespace where all display element are accessible.
+_ROOT = "Dsp"
 
 
 #
 # Class for display windows.
 #
-class Display(Observable):
+class Display(gtk.HBox, Observable):
+
+    __slots__ = ('__sensor_controls', '__arrays', '__sensor_menu',
+                 '__group', '__script',
+                 '__configurator', '__path', '__display_file', '__id',
+                 '__last_pointer_pos', '__pointer_pos', '__is_sensitive',
+                 '__menu')
 
     # observer commands
     OBS_CLOSE = 0
     OBS_RESTART = 1
+    OBS_GEOMETRY = 2
+    OBS_SHAPE = 3
+    OBS_TITLE = 4
+    OBS_ICON = 5
+    OBS_FLAGS = 6
+    OBS_CLOSED = 7
+
+
+    # event types
+    EVENT_MOTION = 0
+    EVENT_PRESS = 1
+    EVENT_RELEASE = 2
+    EVENT_KEY_PRESS = 3
+    EVENT_KEY_RELEASE = 4
+    EVENT_LEAVE = 5
+    EVENT_SCROLL = 6
 
 
     # regular expression to test whether a path is absolute
     __IS_ABSOLUTE_PATH_RE = re.compile("[a-zA-Z]+://.+")
 
-
     def __init__(self, ident, rep):
 
         # the display menu
         self.__DISPLAY_MENU = [
-            (_("_Configure display"), self.__handle_configure),
-            (_("_Move display"), self.__handle_move),
+            (_("_Configure desklet"), self.__handle_configure),
+            (_("_Move desklet"), self.__handle_move),
             ("", None),
             (_("_View Source"), self.__handle_source),
             ("", None),
-            (_("Re_start display"), self.__handle_restart),
-            (_("_Remove display"), self.__handle_remove)]
+            (_("Re_start desklet"), self.__handle_restart),
+            (_("_Remove desklet"), self.__handle_remove)]
 
-        self.__motion_index = 0
-        self.__leave_index = 0
 
         # the controls handling the deprecated sensor stuff: id -> control
         self.__sensor_controls = {}
@@ -63,18 +85,21 @@ class Display(Observable):
         # FIXME: remove eventually
         self.__sensor_menu = []
 
-        # the container of this display
-        self.__container = None
-        self.__container_settings = {}
+        # content of this display (valid until display has been initialized)
+        self.__content = None
+
+        # scriptlets of this display (valid until display has been initialized)
+        self.__scriptlets = []
 
         # the root TargetGroup
         self.__group = None
 
         # the scripting environment
-        self.__script = Script(ident)
+        self.__script = Script(ident, rep)
 
         # the configurator object
-        self.__configurator = None
+        self.__configurator = DisplayConfigger(ident)
+        self.__configurator.set_scripting_environment(self.__script)
 
         # the path of the .display file
         self.__path = os.path.dirname(rep)
@@ -101,43 +126,82 @@ class Display(Observable):
         # the menu to open
         self.__menu = None
 
+        gtk.HBox.__init__(self)
+
+
+
+    #
+    # Returns the unique ID of this display.
+    #
+    def get_id(self): return self.__id
 
     def get_next_child_index(self): return -1
+
     def get_index_path(self): return []
 
 
 
     #
-    # Sets the container of the display.
+    # Initializes the display. Call this after you have finished constructing
+    # it.
     #
-    def set_container(self, container):
+    def initialize(self):
+        assert self.__content
 
-        self.__container = container
-        container.add_observer(self.__on_observe_container)
-        container.add(self.__group.get_widget())
+        childtype, settings, children = self.__content
 
-        for k, v in self.__container_settings.items():
-            self.set_prop(k, v)
-        self.__container_settings.clear()
+        # load saved positions
+        positions = DefaultStateSaver().get_key("positions", {})
+        lx, ly = positions.get(self.get_id(), (UNSET_COORD, UNSET_COORD))
+        x = settings.get("x", lx)
+        y = settings.get("y", ly)
+        if (x != UNSET_COORD): settings["x"] = x
+        if (y != UNSET_COORD): settings["y"] = y
 
-        # set geometry
-        x, y, w, h = self.__group.get_geometry()
-        ux, uy, uw, uh = self.__group.get_user_geometry()
-        if (ux.is_unset() or uy.is_unset()):
-            container.set_position(UNSET_COORD, UNSET_COORD)
-        else:
-            container.set_position(x.as_px(), y.as_px())
-        container.set_size(w.as_px(), h.as_px())
-        container.connect("delete-event", self.__on_close)
+        # build widget hierarchy
+        self.new_child(childtype, settings, children)
+        self.add(self.__group.get_widget())
 
-        container.connect("button-press-event", self.__on_button, False)
-        container.connect("button-release-event", self.__on_button, True)
-        container.connect("key-press-event", self.__on_key, False)
-        container.connect("key-release-event", self.__on_key, True)
-        container.connect("scroll-event", self.__on_scroll)
-        container.connect("motion-notify-event", self.__on_motion, False)
-        container.connect("leave-notify-event", self.__on_motion, True)
+        # run initial scripts
+        for s in self.__scriptlets:
+            self.execute_script(s)
 
+        # load stored configuration
+        self.__configurator.load_config()
+
+
+
+    #
+    # Sets the content of the display.
+    #
+    def set_content(self, childtype, settings, children):
+
+        self.__content = (childtype, settings, children)
+
+
+
+    #
+    # Sets metadata.
+    #
+    def set_metadata(self, author = "", version = "", name= "", preview = ""):
+
+        # a banner without the display's name would look crappy
+        if (name):
+            if (preview): preview = self.get_full_path(preview)
+            self.__configurator.set_banner(preview,
+                                  "<big>%s</big> %s\n"
+                                  "<small>%s</small>" % (name, version, author))
+
+
+
+    #
+    # Adds the given scriptlet.
+    #
+    def add_scriptlet(self, code, filename):
+
+        scriptlet = Scriptlet(code, filename)
+        #self.execute_script(scriptlet)
+        self.__scriptlets.append(scriptlet)
 
 
 
@@ -155,8 +219,9 @@ class Display(Observable):
     #
     def execute_callback_script(self, code, this):
 
-        self.__script.add_target("self", this, True)
-        self.__script.execute(code)
+        scriptlet = Scriptlet(code, self.__display_file)
+        self.__script.add_element(None, "self", this)
+        self.__script.execute(scriptlet)
 
 
 
@@ -165,7 +230,116 @@ class Display(Observable):
     #
     def add_target_to_script(self, name, target):
 
-        self.__script.add_target(name, target)
+        index_path = target.get_index_path()
+        length = len(index_path)
+
+        if ("#" in name): name = name[:name.find("#")]
+        if (length > 0):
+            self.__script.add_element_with_path(_ROOT, name, target, index_path)
+        else:
+            self.__script.add_element(_ROOT, name, target)
+
+
+
+    #
+    # Builds the configurator.
+    #
+    def build_configurator(self, items):
+
+        self.__configurator.build(items)
+        self.__configurator.set_path(self.__path)
+
+
+
+    #
+    # Sends an event to be executed to the display.
+    #
+    def send_event(self, etype, *args):
+
+        if (not self.__is_sensitive): return
+        w, h = self.__group.get_widget().size_request()
+        lx, ly = self.__pointer_pos
+
+
+        if (etype == self.EVENT_MOTION):
+            x, y = args
+
+            utils.request_call(self.__queue_motion, x, y, w, h, False)
+            return
+
+
+        elif (etype == self.EVENT_LEAVE):
+            utils.request_call(self.__queue_motion, 0, 0, w, h, True)
+            return
+
+
+        elif (etype == self.EVENT_PRESS):
+            button, x, y, counter = args
+
+            self.__pointer_pos = (x, y)
+            if (counter == 1):
+                action = DisplayTarget.ACTION_PRESS
+            else:
+                action = DisplayTarget.ACTION_DOUBLECLICK
+
+            event = Struct(button = button, _args = [button])
+
+
+        elif (etype == self.EVENT_RELEASE):
+            button, x, y = args
+
+            if (button == 1):
+                if (abs(lx - x) < 10 and abs(ly - y) < 10):
+                    action = DisplayTarget.ACTION_CLICK
+                else:
+                    action = DisplayTarget.ACTION_RELEASE
+            elif (button == 2):
+                return
+            elif (button == 3):
+                action = DisplayTarget.ACTION_MENU
+            else:
+                return
+            event = Struct(button = button, _args = [button])
+
+
+        elif (etype == self.EVENT_SCROLL):
+            direction, x, y = args
+
+            if (direction == gtk.gdk.SCROLL_UP):
+                direction = 0
+            elif (direction == gtk.gdk.SCROLL_DOWN):
+                direction = 1
+            else:
+                direction = -1
+            action = DisplayTarget.ACTION_SCROLL
+            event = Struct(direction = direction, _args = [direction])
+
+
+        elif (etype == self.EVENT_KEY_PRESS):
+            key, x, y = args
+            action = DisplayTarget.ACTION_KEY_PRESS
+            event = Struct(key = key)
+
+
+        elif (etype == self.EVENT_KEY_RELEASE):
+            key, x, y = args
+            action = DisplayTarget.ACTION_KEY_RELEASE
+            event = Struct(key = key)
+
+
+        else:
+            # what kind of event did we get there?
+            assert (False)  # never reached
+
+        self.__group.handle_action(action,
+                                   Unit.Unit(x, Unit.UNIT_PX),
+                                   Unit.Unit(y, Unit.UNIT_PX),
+                                   event)
+        self.__group.notify_handle_action(True)
+
+        # extend the menu or create one if there's none
+        if (action == DisplayTarget.ACTION_MENU):
+            self.__make_menu(0, 0)
 
 
 
@@ -187,6 +361,7 @@ class Display(Observable):
         return self.__path
 
 
+
     #
     # Returns the full path of the given path which may be relative to the
     # .display file.
@@ -201,6 +376,7 @@ class Display(Observable):
             return os.path.join(self.__path, path)
 
 
+
     #
     # Returns the display.
     #
@@ -211,7 +387,11 @@ class Display(Observable):
     def new_child(self, childtype, settings, children):
 
         import targetregistry
-        self.__group = targetregistry.create(childtype, self)
+        # we don't catch the KeyError here, but in the DisplayFactory
+        try:
+            self.__group = targetregistry.create(childtype, self)
+        except KeyError, exc:
+            log(exc)
         self.__group.get_widget().show()
         cid = settings["id"]
         self.add_target_to_script(cid, self.__group)
@@ -223,20 +403,6 @@ class Display(Observable):
             self.__group.set_xml_prop(key, value)
 
 
-    #
-    # Sets the configurator object for storing and changing settings.
-    #
-    def set_configurator(self, configurator):
-
-        self.__configurator = configurator
-        self.__configurator.set_getter_and_setter_and_caller(
-            self.__script.get_value,
-            self.__script.set_value,
-            self.__script.call_function)
-
-        # init configurator
-        self.__configurator.initialize(self.__id)
-        
 
     #
     # Opens the configuration dialog for this display.
@@ -251,51 +417,66 @@ class Display(Observable):
             # support old deprecated sensor stuff
             from DisplayConfigurator import DisplayConfigurator
             dconf = DisplayConfigurator(configurators)
-            
+
         else:
             self.__configurator.show()
 
 
 
+    #
+    # Saves this display's position.
+    #
+    def __save_position(self):
+
+        x, y, nil, nil = self.__group.get_user_geometry()
+        positions = DefaultStateSaver().get_key("positions", {})
+        positions[self.get_id()] = (x.as_px(), y.as_px())
+        DefaultStateSaver().set_key("positions", positions)
+
+
+
     #
     # Removes this display.
     #
     def remove_display(self):
 
+        # save the display's position
+        self.__save_position()
+
+        self.__configurator.destroy()
+        self.update_observer(self.OBS_CLOSED, self.__id)
         self.drop_observers()
-        self.__container.drop_observers()
+
         self.__script.stop()
         for s in self.__sensor_controls.values():
             s.stop = True
 
-        self.__container.remove(self.__group.get_widget())
         self.__group.delete()
-        self.__container.close()
 
         del self.__sensor_controls
         del self.__mapping
         del self.__group
 
 
+
     #
     # Purges this display.
     #
     def purge_display(self):
 
-        print "Info: Purging ID '%s'." % (self.__id)
-        config  = ConfigManager()
-        config.clear(self.__id)
+        # remove the display's position
+        positions = DefaultStateSaver().get_key("positions", {})
+        try:
+            del positions[self.get_id()]
+        except KeyError:
+            # position wasn't stored
+            pass
 
-        # remove the state file
+        # clean up scripting environment
         self.__script.remove()
 
-
-    #
-    # Reacts on closing the window.
-    #
-    def __on_close(self, src, event):
-
-        self.update_observer(self.OBS_CLOSE, self.__id)
+        # remove the configuration
+        self.__configurator.remove_config()
 
 
 
@@ -309,11 +490,10 @@ class Display(Observable):
 
             # analyze call to see if it's a legacy call
             # FIXME: remove eventually :)
-            import re
             if (re.match("\w+:.*", call)):
-                import warnings
-                warnings.warn("Please use new style call", DeprecationWarning)
-                
+                log("Deprecation: Please use new style call.",
+                    is_warning = True)
+
                 try:
                     legacy_args = event._args
                 except:
@@ -329,6 +509,7 @@ class Display(Observable):
                 self.execute_callback_script(call, src)
 
 
+
     #
     # Has to be called when a menu is to open. Do not open the menu yourself.
     #
@@ -337,87 +518,6 @@ class Display(Observable):
         self.__menu = menu
 
 
-    #
-    # Reacts on keypresses.
-    #
-    def __on_key(self, src, event, is_release):
-
-        key = gtk.gdk.keyval_name(event.keyval)
-            
-        # F10 opens the menu
-        if (key == "F10"):
-            action = DisplayTarget.ACTION_MENU
-            px, py = self.__pointer_pos
-            self.__group.handle_action(action,
-                                       Unit.Unit(px, Unit.UNIT_PX),
-                                       Unit.Unit(py, Unit.UNIT_PX),
-                                       Struct(button = 0,
-                                              _args = [0]))
-            self.__group.notify_handle_action(True)
-            
-            self.__make_menu(0, event.time)
-
-        else:
-            if (is_release): action = DisplayTarget.ACTION_KEY_PRESS
-            else: action = DisplayTarget.ACTION_KEY_RELEASE
-            px, py = self.__container.get_pointer()
-            self.__group.handle_action(action,
-                                       Unit.Unit(px, Unit.UNIT_PX),
-                                       Unit.Unit(py, Unit.UNIT_PX),
-                                       Struct(key = key))
-            self.__group.notify_handle_action(True)
-            
-
-
-    #
-    # Reacts on button events.
-    #
-    def __on_button(self, src, event, is_release = False):
-
-        if (not self.__is_sensitive): return
-
-        px, py = self.__container.get_pointer()
-        lx, ly = self.__pointer_pos
-        button = event.button
-
-        # determine action
-        if (not is_release):
-            self.__pointer_pos = (px, py)
-            if (button == 1 and event.type == gtk.gdk._2BUTTON_PRESS):
-                action = DisplayTarget.ACTION_DOUBLECLICK
-            elif (button == 1):
-                action = DisplayTarget.ACTION_PRESS
-            elif (button == 2):
-                return
-            elif (button == 3):
-                return
-            else:
-                return
-
-        else:
-            if (button == 1):
-                if (abs(lx - px) < 10 and abs(ly - py) < 10):
-                    action = DisplayTarget.ACTION_CLICK
-                else:
-                    action = DisplayTarget.ACTION_RELEASE
-            elif (button == 2):
-                return
-            elif (button == 3):
-                action = DisplayTarget.ACTION_MENU
-            else:
-                return
-
-        self.__group.handle_action(action,
-                                   Unit.Unit(px, Unit.UNIT_PX),
-                                   Unit.Unit(py, Unit.UNIT_PX),
-                                   Struct(button = button,
-                                                          _args = [button]))
-        self.__group.notify_handle_action(True)
-
-        # extend the menu or create one if there's none
-        if (action == DisplayTarget.ACTION_MENU):
-            self.__make_menu(0, event.time)
-
 
     #
     # Builds the menu.
@@ -430,8 +530,8 @@ class Display(Observable):
             self.__menu = self.__group.new_child("menu",
                                                  {"id": ident},
                                                  [])
-            self.__group.set_prop("on-menu", "Dsp.%s.popup()" % (ident))
-            
+            self.__group.set_prop("on-menu", "%s.%s.popup()" % (_ROOT, ident))
+
         elif (not "display" in self.__menu.get_slots()):
             # add separator
             need_separator = True
@@ -479,33 +579,8 @@ class Display(Observable):
 
 
 
+    def __queue_motion(self, px, py, w, h, is_leave):
 
-    #
-    # Reacts on moving the mouse.
-    #
-    def __on_motion(self, src, event, is_leave):
-
-        if (not self.__is_sensitive): return
-        px, py = self.__container.get_pointer()
-        w, h = self.__group.get_widget().size_request()
-
-        if (is_leave):
-            self.__leave_index += 1
-            gtk.idle_add(self.__queue_motion, self.__leave_index,
-                         px, py, w, h, is_leave)
-        else:
-            self.__motion_index += 1
-            gtk.idle_add(self.__queue_motion, self.__motion_index,
-                         px, py, w, h, is_leave)
-
-
-
-    def __queue_motion(self, index, px, py, w, h, is_leave):
-
-        # we can drop events which are not recent
-        if (is_leave and index != self.__leave_index): return
-        elif (not is_leave and index != self.__motion_index): return
-        #if (index != self.__motion_queue_index): return
         # some window managers send a LEAVE event for mouse clicks;
         # work around this
         if (is_leave and (px, py) == self.__last_pointer_pos
@@ -516,7 +591,6 @@ class Display(Observable):
         if (not is_leave and (px, py) == self.__last_pointer_pos): return
         else: self.__last_pointer_pos = (px, py)
 
-        
         if (not is_leave):
             ux = Unit.Unit(px, Unit.UNIT_PX)
             uy = Unit.Unit(py, Unit.UNIT_PX)
@@ -528,54 +602,6 @@ class Display(Observable):
 
 
 
-    #
-    # Reacts on rolling the mouse wheel.
-    #
-    def __on_scroll(self, src, event):
-
-        if (not self.__is_sensitive): return
-
-        px, py = self.__container.get_pointer()
-
-        if (event.direction == gtk.gdk.SCROLL_UP):
-            direction = 0
-        elif (event.direction == gtk.gdk.SCROLL_DOWN):
-            direction = 1
-        else:
-            direction = -1
-
-        self.__group.handle_action(DisplayTarget.ACTION_SCROLL,
-                                   Unit.Unit(px, Unit.UNIT_PX),
-                                   Unit.Unit(py, Unit.UNIT_PX),
-                                   Struct(direction = direction,
-                                          _args = [direction]))
-        self.__group.notify_handle_action(True)
-
-
-
-    #
-    # Observer for the container.
-    #
-    def __on_observe_container(self, src, cmd):
-
-        if (cmd == src.OBS_GEOMETRY):
-            x, y, w, h = src.get_geometry()
-            x = Unit.Unit(x, Unit.UNIT_PX)
-            y = Unit.Unit(y, Unit.UNIT_PX)
-            w = Unit.Unit(w, Unit.UNIT_PX)
-            h = Unit.Unit(h, Unit.UNIT_PX)
-            cx, cy, cw, ch = self.__group.get_geometry()
-
-            if ((cx, cy) != (x, y)):
-                ax, ay = self.__group.get_anchored_coords(x, y, cw, ch)
-                dx, dy= x - ax, y - ay
-                self.set_position(x.as_px() + dx.as_px(),
-                                  y.as_px() + dy.as_px())
-
-        elif (cmd == src.OBS_LOCK): self.set_sensitive(False)
-        elif (cmd == src.OBS_UNLOCK): self.set_sensitive(True)
-
-
     #
     # Observer for the root group.
     #
@@ -585,11 +611,15 @@ class Display(Observable):
             x, y, w, h = self.__group.get_geometry()
             ux, uy = self.__group.get_user_geometry()[:2]
 
-            if (self.__container):
-                self.__container.set_position(x.as_px(), y.as_px())
-
-                if (w != 0 and h != 0):
-                    self.__container.set_size(w.as_px(), h.as_px())
+            if (ux.is_unset() or uy.is_unset()):
+                utils.request_call(self.update_observer, self.OBS_GEOMETRY,
+                                   UNSET_COORD, UNSET_COORD,
+                                   w.as_px(), h.as_px())
+            else:
+                self.__save_position()
+                utils.request_call(self.update_observer, self.OBS_GEOMETRY,
+                                   x.as_px(), y.as_px(),
+                                   w.as_px(), h.as_px())
 
 
 
@@ -598,12 +628,29 @@ class Display(Observable):
     #
     def set_position(self, x = UNSET_COORD, y = UNSET_COORD):
 
-        if (x == y == UNSET_COORD):
-            #self.__container.set_position(x, y)
-            pass
-        else:
-            self.__group.set_xml_prop("x", str(x))
-            self.__group.set_xml_prop("y", str(y))
+        if (x != y != UNSET_COORD):
+            x = Unit.Unit(x, Unit.UNIT_PX)
+            y = Unit.Unit(y, Unit.UNIT_PX)
+            cx, cy, cw, ch = self.__group.get_geometry()
+
+            if ((cx, cy) != (x, y)):
+                ax, ay = self.__group.get_anchored_coords(x, y, cw, ch)
+                dx, dy= x - ax, y - ay
+                new_x = x.as_px() + dx.as_px()
+                new_y = y.as_px() + dy.as_px()
+
+                self.__group.set_xml_prop("x", str(new_x))
+                self.__group.set_xml_prop("y", str(new_y))
+
+
+
+    #
+    # Sets the size of the display.
+    #
+    def set_size(self, width, height):
+
+        self.__group.set_xml_prop("width", str(width))
+        self.__group.set_xml_prop("height", str(height))
 
 
 
@@ -639,40 +686,60 @@ class Display(Observable):
     #
     def set_prop(self, key, value):
 
-        if (not self.__container):
-            # save for later when we have a container
-            self.__container_settings[key] = value
-
-        elif (key == "window-flags"):
-            self.__container.set_window_flags(value)
+        if (key == "window-flags"):
+            self.update_observer(self.OBS_FLAGS, value)
 
         elif (key == "title"):
-            self.__container.set_title(value)
+            self.update_observer(self.OBS_TITLE, value)
 
         elif (key == "icon"):
             filename = self.get_full_path(value)
+            loader = gtk.gdk.PixbufLoader()
             try:
-                loader = gtk.gdk.PixbufLoader()
-                data = gnomevfs.read_entire_file(filename)
-                loader.write(data, len(data))
-                loader.close()
-                pixbuf = loader.get_pixbuf()
-                self.__container.set_icon(pixbuf)
+                data = vfs.read_entire_file(filename)
             except:
-                print "could not set icon ", filename
+                return
+            try:
+                loader.write(data, len(data))
+            except:
+                log("Warning: Invalid image format.")
+                return
+
+            loader.close()
+            pixbuf = loader.get_pixbuf()
+            self.update_observer(self.OBS_ICON, pixbuf)
 
         elif (key == "shape"):
-            filename = self.get_full_path(value)
-            try:
+            if (value.lstrip().startswith("<")):
+                from utils.DOM import DOM
+                from utils import svg
+                w, h, = self.size_request()
+                if (not w or not h): return
+                root = DOM(value).get_root()
+                root["width"] = `w`
+                root["height"] = `h`
+                img = gtk.Image()
+                svg.render(img, w, h, str(root))
+                pixbuf = img.get_pixbuf()
+
+            else:
+                filename = self.get_full_path(value)
                 loader = gtk.gdk.PixbufLoader()
-                data = gnomevfs.read_entire_file(filename)
-                loader.write(data, len(data))
+                try:
+                    data = vfs.read_entire_file(filename)
+                except:
+                    return
+                try:
+                    loader.write(data, len(data))
+                except:
+                    log("Warning: Invalid image format.")
+                    return
+
                 loader.close()
                 pixbuf = loader.get_pixbuf()
-                pix, mask = pixbuf.render_pixmap_and_mask(1)
-                self.__container.set_shape(mask)
-            except:
-                print "could not set shape ", filename
+
+            pix, mask = pixbuf.render_pixmap_and_mask(1)
+            self.update_observer(self.OBS_SHAPE, mask)
 
 
 
@@ -689,6 +756,7 @@ class Display(Observable):
         sensor.bind("menu", set_menu)
 
 
+
     #
     # Binds a sensor's output to an element.
     # FIXME: remove eventually :)
@@ -706,6 +774,7 @@ class Display(Observable):
             self.__mapping[(ident, port)].append((element, prop))
 
 
+
     #
     # Calls a function of a Sensor.
     # FIXME: Remove eventually
@@ -717,18 +786,15 @@ class Display(Observable):
         for ident, callname, userargs in cmd:
             sensorctrl = self.__sensor_controls[ident]
 
-            #sensor = self.__get_sensor(ident)
             allargs = args + userargs
 
             # the sensor is an external module, so we make sure it cannot crash
             # the application
             try:
-                #sensor.send_action(callname, path, allargs)
                 sensorctrl.action = (callname, path, allargs)
 
             except StandardError, exc:
-                import traceback; traceback.print_exc()
-                print >>sys.stderr, "The sensor produced an error: %s" % (exc)
+                log("The sensor produced an error: %s" % (exc,))
 
 
 
@@ -745,6 +811,7 @@ class Display(Observable):
             pass
 
 
+
     #
     # Registers an array to be parent array of the given child.
     # FIXME: remove eventually :)
@@ -770,19 +837,28 @@ class Display(Observable):
             return (Unit.ZERO, Unit.ZERO, Unit.ZERO, Unit.ZERO)
 
 
+
     def __handle_configure(self, *args):
 
         self.__open_configurator()
 
 
+
     def __handle_move(self, *args):
 
-        self.__container.set_position(UNSET_COORD, UNSET_COORD)
+        self.update_observer(self.OBS_GEOMETRY, UNSET_COORD, UNSET_COORD, 0, 0)
+
 
 
     def __handle_source(self, *args):
 
-        os.system("gnome-text-editor %s & disown" % self.__display_file)
+        from config import settings
+
+        if (HAVE_WIN32):
+            os.system("start notepad \"%s\"" % (self.__display_file,))
+        else:
+            os.system("%s \"%s\" & disown" % (settings.editor,
+                                              self.__display_file))
 
 
     def __handle_restart(self, *args):
@@ -790,20 +866,29 @@ class Display(Observable):
         self.update_observer(self.OBS_RESTART, self.__id)
 
 
+
     def __handle_remove(self, *args):
 
-        def remove_display(*args): self.update_observer(self.OBS_CLOSE, self.__id)
-        
+        self.close()
+
+
+
+    def close(self):
+
+        def remove_display(*args):
+            self.update_observer(self.OBS_CLOSE, self.__id)
+
 
         utils.dialog.question(None,
-                              _("Do you really want to remove this display ?"),
-                              _("This display will no longer be displayed "
+                              _("Do you really want to remove this desklet ?"),
+                              _("This desklet will no longer be displayed "
                                 "and its configuration will be purged."),
                               (gtk.STOCK_CANCEL, Null),
                               (gtk.STOCK_DELETE, remove_display)
-                              )
+                             )
+
+
 
-    
     #
     # Returns a unique ID string.
     #
