@@ -3,8 +3,9 @@ from utils.Struct import Struct
 from utils import Unit
 from DataTarget import DataTarget
 
-from urllib import unquote # for filename conversion
+from urllib import unquote
 
+import gobject
 import gtk
 
 
@@ -13,6 +14,10 @@ import gtk
 #
 class DisplayTarget(DataTarget):
 
+    __slots__ = ('__has_adjust_request', '__geometry', '__old_geometry',
+                 '__user_geometry', '__relative', '__is_relative', '__anchor',
+                 '__had_action')
+
     # observer actions
     OBS_GEOMETRY = 0
 
@@ -48,7 +53,10 @@ class DisplayTarget(DataTarget):
     __DND_LINK = [("x-url/http", 0, 0), ("_NETSCAPE_URL", 0, 0)]
 
 
-    def __init__(self, parent):
+    def __init__(self, name, parent):
+
+        # flag indicating whether an geometry adjust has been requested already
+        self.__has_adjust_request = False
 
         # the real coordinates and size of this widget
         self.__geometry = (Unit.ZERO, Unit.ZERO,
@@ -72,7 +80,7 @@ class DisplayTarget(DataTarget):
         # the value of the last notify_handle_action call
         self.__had_action = False
 
-        DataTarget.__init__(self, parent)
+        DataTarget.__init__(self, name, parent)
 
         for prop, datatype in [("x", TYPE_UNIT),
                                ("y", TYPE_UNIT),
@@ -80,7 +88,6 @@ class DisplayTarget(DataTarget):
                                ("height", TYPE_UNIT)]:
             self._register_property(prop, datatype,
                                     self._setp_geometry, self._getp_geometry)
-        #end for
 
         self._register_property("relative-to", TYPE_LIST,
                                 self._setp_relative_to, self._getp)
@@ -106,10 +113,7 @@ class DisplayTarget(DataTarget):
                        self.ACTION_KEY_PRESS,
                        self.ACTION_KEY_RELEASE):
             self._register_action(action)
-        #end for
 
-        #gtk.idle_add(gtk.Widget.show, self.get_widget())
-        #self.get_widget().set_double_buffered(False)
 
 
     #
@@ -177,14 +181,14 @@ class DisplayTarget(DataTarget):
         if (anchor in (self.ANCHOR_NW, self.ANCHOR_W, self.ANCHOR_SW)):
             ax = x
         elif (anchor in (self.ANCHOR_N, self.ANCHOR_CENTER, self.ANCHOR_S)):
-            ax = x - Unit.Unit(w.as_pt() / 2, Unit.UNIT_PT)
+            ax = x - (w / 2)
         else:
             ax = x - w
 
         if (anchor in (self.ANCHOR_NW, self.ANCHOR_N, self.ANCHOR_NE)):
             ay = y
         elif (anchor in (self.ANCHOR_W, self.ANCHOR_CENTER, self.ANCHOR_E)):
-            ay = y - Unit.Unit(h.as_pt() / 2, Unit.UNIT_PT)
+            ay = y - (h / 2)
         else:
             ay = y - h
 
@@ -272,12 +276,25 @@ class DisplayTarget(DataTarget):
         return self.__user_geometry
 
 
+    #
+    # Requests a geometry adjustment. Use this to avoid redundant calls of
+    # adjust_geometry().
+    #
+    def request_adjust_geometry(self):
+
+        if (not self.__has_adjust_request):
+            self.__has_adjust_request = True
+            gobject.timeout_add(0, self.adjust_geometry)
+
+
 
     #
     # Adjusts the geometry of this target.
     #
     def adjust_geometry(self):
 
+        self.__has_adjust_request = False
+
         x, y, w, h = self.__geometry
         ux, uy, uw, uh = self.get_user_geometry()
         anchor_x, anchor_y = self.get_anchor()
@@ -293,16 +310,16 @@ class DisplayTarget(DataTarget):
 
         # compute geometry in pixels
         if (ux.is_unset()): new_x = x + anchor_x
-        else: new_x = ux #.as_px()
+        else: new_x = ux
 
         if (uy.is_unset()): new_y = y + anchor_y
-        else: new_y = uy #.as_px()
+        else: new_y = uy
 
         if (uw.is_unset()): new_w = self._get_value_for_unset(width = w)
-        else: new_w = uw #.as_px()
+        else: new_w = uw
 
         if (uh.is_unset()): new_h = self._get_value_for_unset(height = h)
-        else: new_h = uh #.as_px()
+        else: new_h = uh
 
         # handle relative positioning
         if (self.__relative):
@@ -325,11 +342,11 @@ class DisplayTarget(DataTarget):
             if (is_rel_y): new_y += th
         #end if
 
-
         # get coords of the top left corner
         new_x, new_y = self.get_anchored_coords(new_x, new_y, new_w, new_h)
-        if ((parent == self._get_display() and self.get_widget().window)
+        if (parent == self._get_display()
               or (new_x, new_y, new_w, new_h) != self.__old_geometry):
+
             self.__geometry = (new_x, new_y, new_w, new_h)
             self.__old_geometry = (new_x.copy(), new_y.copy(), new_w.copy(), new_h.copy())
             self.get_widget().set_size_request(new_w.as_px(), new_h.as_px())
@@ -359,7 +376,8 @@ class DisplayTarget(DataTarget):
     def _propagate_geometry(self):
 
         parent = self._get_parent()
-        if (parent): gtk.timeout_add(0, parent.adjust_geometry)
+        if (parent):
+            parent.request_adjust_geometry()
         self.update_observer(self.OBS_GEOMETRY)
 
 
@@ -487,21 +505,21 @@ class DisplayTarget(DataTarget):
     def _getp_geometry(self, key):
 
         x, y, w, h = self.get_geometry()
-        x, y = self.get_anchored_coords(x, y, w, h)
+        #x, y = self.get_anchored_coords(x, y, w, h)
 
         parent_width, parent_height = self._get_parent_size()
 
         if (key == "x"):
-            unit = x #Unit.Unit(x, Unit.UNIT_PX)
+            unit = x
             unit.set_100_percent(parent_width.as_px())
         elif (key == "y"):
-            unit = y #Unit.Unit(y, Unit.UNIT_PX)
+            unit = y
             unit.set_100_percent(parent_height.as_px())
         elif (key == "width"):
-            unit = w #Unit.Unit(w, Unit.UNIT_PX)
+            unit = w
             unit.set_100_percent(parent_width.as_px())
         elif (key == "height"):
-            unit = h #Unit.Unit(h, Unit.UNIT_PX)
+            unit = h
             unit.set_100_percent(parent_height.as_px())
 
         return unit
@@ -539,3 +557,4 @@ class DisplayTarget(DataTarget):
                                             gtk.gdk.ACTION_COPY)
             self.get_widget().connect("drag_data_received",
                                       self.__on_link_drop)
+
