@@ -11,12 +11,11 @@ import gtk
 #
 class TargetAlignment(ContainerTarget):
 
-    def __init__(self, parent):
+    def __init__(self, name, parent):
 
         self.__alignment = (0.0, 0.0)
 
-
-        ContainerTarget.__init__(self, parent)
+        ContainerTarget.__init__(self, name, parent)
         self.__align = gtk.Alignment(0.5, 1, 0, 0)
         self.__align.show()
 
