@@ -1,7 +1,7 @@
 from ContainerTarget import ContainerTarget
+from Layouter import Layouter, LayoutError
 from utils.datatypes import *
 from utils import Unit
-import layouters
 
 import gtk
 
@@ -11,7 +11,7 @@ import gtk
 #
 class TargetArray(ContainerTarget):
 
-    def __init__(self, parent):
+    def __init__(self, name, parent):
 
         # the current layouter function
         self.__layouter = []
@@ -31,7 +31,7 @@ class TargetArray(ContainerTarget):
         # mapping between unique IDs and targets
         self.__targets = {}
 
-        ContainerTarget.__init__(self, parent)
+        ContainerTarget.__init__(self, name, parent)
         self.__layout = gtk.Fixed()
         self.__layout.show()
 
@@ -50,7 +50,7 @@ class TargetArray(ContainerTarget):
 
 
     #
-    # Returns last child inxex
+    # Returns next child index
     #
     def get_next_child_index(self): return len(self.__children)
 
@@ -64,6 +64,7 @@ class TargetArray(ContainerTarget):
 
         # the array should be empty at the beginning but we need to create a
         # child so the array can connect to sensor output
+        # remove eventually
         self.__add_child()
         self.__remove_child()
 
@@ -121,27 +122,23 @@ class TargetArray(ContainerTarget):
         layout = parts[0]
         args = parts[1:]
 
-        # get the layouter function
-        layouter = layouters.LAYOUTERS[layout]
-
+        try:
+            layouter = Layouter(layout)
+        except LayoutError:
+            return
 
         cnt = 0
         previous_id = ""
         for child in self.__children:
             x, y, w, h = child.get_geometry()
-            cx, cy, relative = layouter(cnt, args)
+            rel, cx, cy = layouter.layout(cnt, args)
             cx = Unit.Unit(cx, Unit.UNIT_PX)
             cy = Unit.Unit(cy, Unit.UNIT_PX)
 
             cx, cy = child.get_anchored_coords(cx, cy, w, h)
 
-            # handle relative positioning for dynamic sized children
-            rel = ""
-            if (relative == layouters.RELATIVE_X): rel = "x"
-            elif (relative == layouters.RELATIVE_Y): rel = "y"
-            elif (relative == layouters.RELATIVE_XY): rel = "xy"
-
-            if (not rel): child.set_position(cx, cy)
+            if (not rel):
+                child.set_position(cx, cy)
 
             if (previous_id and rel):
                 child.set_prop("relative-to", (previous_id, rel))
@@ -149,7 +146,7 @@ class TargetArray(ContainerTarget):
             previous_id = self.get_id_by_child(child)
             cnt += 1
 
-        self.adjust_geometry()
+        self.request_adjust_geometry()
 
 
 
@@ -200,3 +197,4 @@ class TargetArray(ContainerTarget):
         self.__layouter = value
         self.__place_children()
         self._setp(key, value)
+
