@@ -2,8 +2,6 @@ from DisplayTarget import DisplayTarget
 from utils.datatypes import *
 
 import gtk
-import bonobo
-import bonobo.ui
 
 
 #
@@ -11,7 +9,7 @@ import bonobo.ui
 #
 class TargetBonoboControl(DisplayTarget):
 
-    def __init__(self, parent):
+    def __init__(self, name, parent):
 
         # the control; you can load a control only once
         self.__control = None
@@ -19,7 +17,7 @@ class TargetBonoboControl(DisplayTarget):
         self.__widget = gtk.HBox()
         self.__widget.show()
 
-        DisplayTarget.__init__(self, parent)
+        DisplayTarget.__init__(self, name, parent)
 
         self._register_property("oafiid", TYPE_STRING,
                                 self._setp_oafiid, self._getp)
@@ -30,6 +28,7 @@ class TargetBonoboControl(DisplayTarget):
 
     def _setp_oafiid(self, key, value):
 
+        import bonobo.ui
         try:
             container = bonobo.ui.Container()
             control = bonobo.ui.Widget(str(value),
@@ -47,7 +46,6 @@ class TargetBonoboControl(DisplayTarget):
             self._setp(key, value)
 
         except StandardError, exc:
-            import sys
-            import traceback; traceback.print_exc()
-            print >>sys.stderr, \
-                "An error occured while setting the oafiid: %s" % (exc)
+            log("Warning: An error occured while setting the oafiid:\n%s" \
+                % (exc,))
+
