@@ -2,14 +2,11 @@ from ContainerTarget import ContainerTarget
 from utils.datatypes import *
 from utils.Struct import Struct
 from utils import Unit
+from utils import vfs
 
 import gtk
 import os
 
-try:
-    import gnomevfs
-except ImportError:
-    import gnome.vfs as gnomevfs
 
 _STATE_VISIBLE = 0
 _STATE_HIDDEN = 1
@@ -22,11 +19,11 @@ class TargetExpander(ContainerTarget):
 
     __ACTION_EXPAND = "expand"
 
-    def __init__(self, parent):
+    def __init__(self, name, parent):
 
         self.__show_state = _STATE_VISIBLE
 
-        ContainerTarget.__init__(self, parent)
+        ContainerTarget.__init__(self, name, parent)
         self.__widget = gtk.Table(2, 2)
         self.__widget.show()
 
@@ -57,7 +54,6 @@ class TargetExpander(ContainerTarget):
             self.__set_expanded(True)
 
 
-
     def __set_expanded(self, value):
 
         if (value): self.__show_state = _STATE_VISIBLE
@@ -80,21 +76,24 @@ class TargetExpander(ContainerTarget):
                                                _args = [expand]))
 
 
-
     def __set_icon(self, uri):
 
+        if (not uri): return
         uri = self._get_display().get_full_path(uri)
+        loader = gtk.gdk.PixbufLoader()
         try:
-            loader = gtk.gdk.PixbufLoader()
-            data = gnomevfs.read_entire_file(uri)
-            loader.write(data, len(data))
-            loader.close()
-            pbuf = loader.get_pixbuf()
-            self.__icon.set_from_pixbuf(pbuf)
-
+            data = vfs.read_entire_file(uri)
         except:
-            import sys
-            print >>sys.stderr, "Could not set icon %s." % (uri)
+            return
+        try:
+            loader.write(data, len(data))
+        except:
+            log("Invalid image format.")
+            return
+
+        loader.close()
+        pbuf = loader.get_pixbuf()
+        self.__icon.set_from_pixbuf(pbuf)
 
 
 
@@ -107,7 +106,6 @@ class TargetExpander(ContainerTarget):
         else:
             return (Unit.Unit(w, Unit.UNIT_PX), Unit.Unit(h, Unit.UNIT_PX),
                     Unit.ZERO, Unit.ZERO)
-        
 
 
     def _setp_expanded(self, key, value):
@@ -116,14 +114,12 @@ class TargetExpander(ContainerTarget):
         self._setp(key, value)
 
 
-
     def _setp_icon(self, key, value):
 
         self.__set_icon(value)
         self._setp(key, value)
 
 
-
     def get_container_geometry(self):
 
         if (self._get_child()):
@@ -135,22 +131,17 @@ class TargetExpander(ContainerTarget):
         return (x, y, w, h)
 
 
-
     def handle_action(self, action, px, py, args):
 
         ContainerTarget.handle_action(self, action, px, py, args)
 
         w, h = self.__icon.size_request()
-        if (action == "click"): print px, w, py, h
         if (0 <= px.as_px() <= w and 0 <= py.as_px() <= h):
             if (action == self.ACTION_CLICK):
                 if (self.__show_state == _STATE_VISIBLE):
                     self.__set_expanded(False)
                 else:
                     self.__set_expanded(True)
-            #end if
-        #end if
-
 
 
     def new_child(self, childtype, settings, children):
