@@ -12,7 +12,7 @@ import gtk
 #
 class TargetFrame(ContainerTarget):
 
-    def __init__(self, parent):
+    def __init__(self, name, parent):
 
         self.__size = (0, 0)
         self.__inner_size = (0, 0)
@@ -20,7 +20,7 @@ class TargetFrame(ContainerTarget):
         self.__images = []
 
 
-        ContainerTarget.__init__(self, parent)
+        ContainerTarget.__init__(self, name, parent)
         self.__table = gtk.Table(3, 3)
         self.__table.show()
 
@@ -31,7 +31,6 @@ class TargetFrame(ContainerTarget):
             img.show()
             self.__table.attach(img, x, x + 1, y, y + 1)
             self.__images.append(img)
-        #end for
 
         self.__box = gtk.HBox()
         self.__box.show()
@@ -57,8 +56,6 @@ class TargetFrame(ContainerTarget):
 
         child = ContainerTarget.new_child(self, childtype, settings, children)
         self.__box.add(child.get_widget())
-        #self._get_child().add_observer(self.__on_resize)
-        #self._init_children()
         self.__redraw_frame()
 
 
@@ -84,6 +81,11 @@ class TargetFrame(ContainerTarget):
             cnt = 0
             for img in self.__images:
                 iw, ih = sizes[cnt]
+                
+                # hide the element if the size is zero
+                if (iw == 0 or ih == 0): img.hide()
+                else: img.show()
+                
                 img.tile(max(iw, 1), max(1, ih))
                 cnt += 1
             #end for
@@ -157,8 +159,6 @@ class TargetFrame(ContainerTarget):
         return (Unit.Unit(w1, Unit.UNIT_PX), Unit.Unit(h1, Unit.UNIT_PX),
                 Unit.Unit(w2, Unit.UNIT_PX), Unit.Unit(h2, Unit.UNIT_PX))
 
-        #return self.__frame_widths[:]
-
 
 
     def get_container_geometry(self):
@@ -175,7 +175,8 @@ class TargetFrame(ContainerTarget):
         h -= h1 + h2
 
         return (Unit.Unit(x, Unit.UNIT_PX), Unit.Unit(y, Unit.UNIT_PX),
-                Unit.Unit(max(w, 1), Unit.UNIT_PX), Unit.Unit(max(h, 1), Unit.UNIT_PX))
+                Unit.Unit(max(w, 1), Unit.UNIT_PX),
+                Unit.Unit(max(h, 1), Unit.UNIT_PX))
 
 
 
@@ -197,8 +198,6 @@ class TargetFrame(ContainerTarget):
         else:
             return []
 
-        #end if
-
 
 
     def _setp_border_uris(self, key, value):
@@ -207,12 +206,14 @@ class TargetFrame(ContainerTarget):
         self._setp(key, value)
 
 
+
     def _setp_border_color(self, key, value):
 
         self.__set_border_color(value)
         self._setp(key, value)
 
 
+
     def _setp_border_width(self, key, value):
 
         self.__set_border_width(value)
