@@ -14,10 +14,9 @@ class TargetGauge(TargetGroup):
     __VERTICAL = "vertical"
 
 
-    def __init__(self, parent):
-
-        TargetGroup.__init__(self, parent)
+    def __init__(self, name, parent):
 
+        TargetGroup.__init__(self, name, parent)
 
         self._register_property("fill", TYPE_FLOAT,
                                 self._setp_fill, self._getp)
@@ -33,7 +32,6 @@ class TargetGauge(TargetGroup):
         child.add_observer(self.__on_observe_child)
 
 
-
     def __on_observe_child(self, src, cmd, *data):
 
         if (cmd == src.OBS_GEOMETRY):
@@ -45,6 +43,7 @@ class TargetGauge(TargetGroup):
     #
     def _setp_fill(self, key, value):
 
+        value = max(0, min(value, 100))
         x, y, width, height = self._get_child().get_geometry()
         width = width.as_pt()
         height = height.as_pt()
@@ -59,7 +58,6 @@ class TargetGauge(TargetGroup):
         self._setp(key, value)
 
 
-
     #
     # "orientation" property.
     #
