@@ -5,17 +5,13 @@ from utils.datatypes import *
 import gtk
 import os
 
-try:
-    import gnomevfs
-except ImportError:
-    import gnome.vfs as gnomevfs
 
 #
 # Class for grouping different targets together.
 #
 class TargetGroup(ContainerTarget):
 
-    def __init__(self, parent):
+    def __init__(self, name, parent):
 
         # the background pixmap
         self.__pixbuf = None
@@ -26,12 +22,10 @@ class TargetGroup(ContainerTarget):
         # the maximum size values of all the children
         self.__max_size = (0, 0)
 
-        ContainerTarget.__init__(self, parent)
+        ContainerTarget.__init__(self, name, parent)
         self.__layout = gtk.Fixed()
         self.__layout.show()
 
-        #self.add(self.__layout)
-
         self.__image = TilingImage()
         self.__image.show()
         self.__layout.put(self.__image, 0, 0)
@@ -48,8 +42,9 @@ class TargetGroup(ContainerTarget):
                                 self._setp_bg_uri, self._getp)
         self._register_property("bg-color", TYPE_STRING,
                                 self._setp_bg_color, self._getp)
+        self._register_property("path", TYPE_STRING, None, self._getp)
+        self._setp("path", self._get_display().get_path())
 
-        #self.connect("expose-event", self.__on_expose)
         self.add_observer(self.__on_observe_size)
 
         self._setp_container_stuff("window-flags", ["managed", "decorated"])
@@ -109,14 +104,8 @@ class TargetGroup(ContainerTarget):
             self.__set_color("#00000000")
             return
 
-        uri = self._get_display().get_full_path(uri)
-        if (gnomevfs.exists(uri)):
-            self.__image.set_from_file(uri)
-
-        else:
-            self.__set_color("#ffffffa0")
-
-        #self.__image.set_size_request(width, height)
+        #uri = self._get_display().get_full_path(uri)
+        self.__image.set_from_file(uri)
 
 
 
@@ -136,7 +125,8 @@ class TargetGroup(ContainerTarget):
     #
     def _setp_bg_uri(self, key, value):
 
-        self.__set_background(value)
+        path = self._get_display().get_full_path(value)
+        self.__set_background(path)
         self._setp(key, value)
 
 
@@ -148,3 +138,4 @@ class TargetGroup(ContainerTarget):
 
         self.__set_color(value)
         self._setp(key, value)
+
