@@ -1,15 +1,13 @@
 from DisplayTarget import DisplayTarget
 from utils.datatypes import *
+from utils import vfs
 
+import gobject
 import gtk
 import gtkhtml2
 import thread
 import urlparse
 
-try:
-    import gnomevfs
-except ImportError:
-    import gnome.vfs as gnomevfs
 
 
 #
@@ -17,12 +15,12 @@ except ImportError:
 #
 class TargetHTML(DisplayTarget):
 
-    def __init__(self, parent):
+    def __init__(self, name, parent):
 
         self.__current_url = ""
         self.__is_loading = False
 
-        DisplayTarget.__init__(self, parent)
+        DisplayTarget.__init__(self, name, parent)
 
         self.__document = gtkhtml2.Document()
         self.__document.connect("request_url", self.__on_request_url)
@@ -56,25 +54,26 @@ class TargetHTML(DisplayTarget):
     #
     def __download_uri(self, uri, handler):
 
+        if (gtk.pygtk_version[1] < 5 and gtk.pygtk_version[2] < 3):
+            read_entire_file = vfs.read_entire_file_win32
+        else:
+            read_entire_file = vfs.read_entire_file
+
         def f(uri, handler):
             self.__is_loading = True
             try:
                 uri = self.__resolve_uri(uri)
-                # up to version 2.6.2 gnome.vfs has a threading bug
-                import gnome
-                if (gnome.gnome_python_version[1] > 6 or
-                    gnome.gnome_python_version[2] > 1):
-                    data = gnomevfs.read_entire_file(uri)
-                else:
-                    import urllib
-                    data = urllib.FancyURLopener().open(uri).read()
+                try:
+                    data = read_entire_file(uri)
+                except:
+                    return
             except:
                 data = "<html><body><h1>%s</h1>The given URL could not be retrieved." \
                        "</body></html>" % uri
 
             self.__is_loading = False
-            gtk.timeout_add(0, handler, data)
-                
+            gobject.timeout_add(0, handler, data)
+
         # download in the background to avoid blocking the program
         thread.start_new_thread(f, (uri, handler))
 
@@ -109,7 +108,7 @@ class TargetHTML(DisplayTarget):
     def __stream_cancel(self, *args): pass
 
 
-    
+
     #
     # Reacts on requesting an URL.
     #
@@ -127,13 +126,11 @@ class TargetHTML(DisplayTarget):
     def __on_link_clicked(self, document, link):
 
         try:
-            url = self.__resolve_uri(link)
-            self.__current_url = url
-            self.__download_uri(url, self.__load_html)
+            self.__current_url = link
+            self.__download_uri(link, self.__load_html)
 
         except:
-            import sys
-            print >> sys.stderr, "Could not resolve %s." % (link,)
+            log("Could not resolve %s." % (link,))
 
 
 
