@@ -1,14 +1,13 @@
 from DisplayTarget import DisplayTarget
 from utils.datatypes import *
 from utils import Unit
+from utils import vfs
+from utils import tiling
+import utils
 
 import gtk
 import os
 
-try:
-    import gnomevfs
-except ImportError:
-    import gnome.vfs as gnomevfs
 
 
 #
@@ -16,13 +15,12 @@ except ImportError:
 #
 class TargetImage(DisplayTarget):
 
-    def __init__(self, parent):
+    def __init__(self, name, parent):
 
         # the pixbuf of the original image and its URI
         self.__original_image = None
-        self.__uri = ""
 
-        self.__parent_geometry = (0, 0, 0, 0)
+        self.__parent_geometry = (Unit.ZERO, Unit.ZERO, Unit.ZERO, Unit.ZERO)
 
         # the original image size
         self.__original_width = 0
@@ -35,7 +33,11 @@ class TargetImage(DisplayTarget):
         # the current saturation value
         self.__current_saturation = 0.0
 
-        DisplayTarget.__init__(self, parent)
+        # the current opacity value
+        self.__current_opacity = 0.0
+        
+
+        DisplayTarget.__init__(self, name, parent)
 
         self.__widget = gtk.Image()
         self.__widget.show()
@@ -46,6 +48,8 @@ class TargetImage(DisplayTarget):
                                 self._setp_saturation, self._getp)
         self._register_property("scale", TYPE_FLOAT,
                                 self._setp_scale, self._getp)
+        self._register_property("opacity", TYPE_FLOAT,
+                                self._setp_opacity, self._getp)
         self._register_property("image-width", TYPE_UNIT,
                                 self._setp_image_size, self._getp_image_size)
         self._register_property("image-height", TYPE_UNIT,
@@ -53,6 +57,7 @@ class TargetImage(DisplayTarget):
 
         self._setp("image-width", Unit.Unit(-1, Unit.UNIT_PX))
         self._setp("image-height", Unit.Unit(-1, Unit.UNIT_PX))
+        self._setp("opacity", 1.0)
         self._setp("saturation", 1.0)
         self._setp("scale", 1.0)
 
@@ -81,21 +86,23 @@ class TargetImage(DisplayTarget):
     def __load_image(self, uri):
 
         if (not uri): return
-        uri = self._get_display().get_full_path(uri)
-        if (uri != self.__uri):
-            try:
-                data = gnomevfs.read_entire_file(uri)
-            except gnomevfs.Error:
-                import sys
-                print >>sys.stderr, "Could not open image %s." % (uri)
-                return
 
-            loader = gtk.gdk.PixbufLoader()
-            loader.write(data, len(data))
-            loader.close()
-            self.__original_image = loader.get_pixbuf()
-            self.__widget.set_from_pixbuf(self.__original_image)
-            self.__uri = uri
+        try:
+            data = vfs.read_entire_file(uri)
+        except:
+            return
+
+        loader = gtk.gdk.PixbufLoader()
+        try:
+           loader.write(data, len(data))
+        except:
+            log("Invalid image format.")
+            return
+
+        loader.close()
+
+        self.__original_image = loader.get_pixbuf()
+        self.__widget.set_from_pixbuf(self.__original_image)
 
         if (self.__original_image):
             self.__original_width = self.__original_image.get_width()
@@ -103,6 +110,11 @@ class TargetImage(DisplayTarget):
             self.__current_width = 0
             self.__current_height = 0
 
+        # clean up dead pixbufs
+        import gc
+        utils.request_call(gc.collect)
+
+
     #
     # Renders the image.
     #
@@ -139,33 +151,29 @@ class TargetImage(DisplayTarget):
 
         # apply the "scale" property
         scale = self.get_prop("scale")
+        # some authors don't pay attention, let's catch it
+        if (scale <= 0.0): scale = 1.0
         imgwidth = int(imgwidth * scale)
         imgheight = int(imgheight * scale)
 
         saturation = self.get_prop("saturation")
+        opacity = self.get_prop("opacity")
 
-        if ((imgwidth, imgheight, saturation) ==
+        if ((imgwidth, imgheight, saturation, opacity) ==
             (self.__current_width, self.__current_height,
-             self.__current_saturation)): return
+             self.__current_saturation, self.__current_opacity)): return
 
-        if ((imgwidth, imgheight) != (self.__original_width,
-                                      self.__original_height)):
-            img = self.__original_image.scale_simple(imgwidth, imgheight, 3)
-        else:
-            img = self.__original_image.copy()
-            
-        if (abs(saturation - 1.0) > 0.00001):
-            simg = img.copy()
-            img.saturate_and_pixelate(simg, saturation, False)
-        else:
-            simg = img
+        # TODO: reimplement saturation again (in C)
+    
+        tiling.render_to_image(self.__widget, self.__original_image.copy(),
+                               imgwidth, imgheight, opacity);
 
-        self.__widget.set_from_pixbuf(simg)
         self.set_size(Unit.Unit(imgwidth, Unit.UNIT_PX),
                       Unit.Unit(imgheight, Unit.UNIT_PX))
         self.__current_width = imgwidth
         self.__current_height = imgheight
         self.__current_saturation = saturation
+        self.__current_opacity = opacity
 
 
     #
@@ -173,17 +181,28 @@ class TargetImage(DisplayTarget):
     #
     def _setp_uri(self, key, value):
 
+        path = self._get_display().get_full_path(value)
         try:
-            self.__load_image(value)
-            self.__render_image()
+            self.__load_image(path)
+
+        except:
+            log("Failed to set URI %s for key %s." % (path, key))
+            return
+
+        self.__render_image()
+        self._setp(key, value)
+
+
+    #
+    # "opacity" property.
+    #
+    def _setp_opacity(self, key, value):
+
+        value = min(1.0, max(0.0, value))
+        old_opac = self.get_prop("opacity")
+        if (abs(value - old_opac) > 0.01):
             self._setp(key, value)
-
-        except StandardError, exc:
-            import sys
-            print >>sys.stderr, \
-                  "Failed to set URI %s for key %s." % (value, key)
-
-
+            self.__render_image()
 
     #
     # "saturation" property.
