@@ -2,7 +2,9 @@ from DisplayTarget import DisplayTarget
 from utils.datatypes import *
 from utils import Unit
 
-import gtk, pango
+import gobject
+import gtk
+import pango
 import re
 
 
@@ -12,8 +14,7 @@ import commands
 fail, CHARMAP = commands.getstatusoutput("locale charmap")
 
 if (fail or not CHARMAP):
-    import sys
-    print >>sys.stderr, "Could not detect character encoding."
+    log("Could not detect character encoding.")
     CHARMAP = "ASCII"
 
 
@@ -31,8 +32,7 @@ class TargetLabel(DisplayTarget):
             try:
                 value = value.encode("UTF-8", "replace")
             except ValueError:
-                import sys
-                print >>sys.stderr, "Unicode -> UTF-8 convertion failed !!!"
+                log("Unicode -> UTF-8 convertion failed !!!")
                 raise
 
         elif (not isinstance(value, str)):
@@ -55,8 +55,7 @@ class TargetLabel(DisplayTarget):
             try:
                 value = value.encode("UTF-8", "replace")
             except ValueError:
-                import sys
-                print >>sys.stderr, "Unicode -> UTF-8 convertion failed !!!"
+                log("Unicode -> UTF-8 convertion failed !!!")
                 raise
 
         else:
@@ -76,13 +75,13 @@ class TargetLabel(DisplayTarget):
 
 
 
-    def __init__(self, parent):
+    def __init__(self, name, parent):
 
         self.__old_value = ""
         self.__font_description = None
         self.__wrap_at = -1
 
-        DisplayTarget.__init__(self, parent)
+        DisplayTarget.__init__(self, name, parent)
 
         self.__widget = gtk.Label("")
         self.__widget.set_line_wrap(False)
@@ -90,8 +89,9 @@ class TargetLabel(DisplayTarget):
              self.__widget.get_layout().get_context().get_font_description()
         self.__widget.show()
 
-
-        self._register_property("value", TYPE_STRING,
+        # i guess a label has to accept everything printable for its value
+        # these days
+        self._register_property("value", TYPE_ANY,
                                 self._setp_value, self._getp)
         self._register_property("color", TYPE_STRING,
                                 self._setp_color, self._getp)
@@ -133,9 +133,9 @@ class TargetLabel(DisplayTarget):
 
         self.__widget.set_size_request(width, height)
         self.__widget.set_markup(value)
-        gtk.timeout_add(0, self.set_size,
-                        Unit.Unit(width, Unit.UNIT_PX),
-                        Unit.Unit(height, Unit.UNIT_PX))
+        gobject.timeout_add(0, self.set_size,
+                            Unit.Unit(width, Unit.UNIT_PX),
+                            Unit.Unit(height, Unit.UNIT_PX))
         self.__old_value = value
 
 
@@ -151,14 +151,12 @@ class TargetLabel(DisplayTarget):
                 if (unit == Unit.UNIT_PERCENT):
                     height = self._get_parent().get_geometry()[3]
                     # don't allow 0 pixels height
-                    size = max(5, height.as_pt() * (size / 100.0))
+                    size = max(1, height.as_pt() * (size / 100.0))
                     unit = Unit.UNIT_PT
                 u = Unit.Unit(size, unit)
                 size = u.as_pt()
                 font = "%s %f" % (name, size)
 
-        #end if
-
         self.__font_description = pango.FontDescription(font)
         self.__widget.modify_font(self.__font_description)
         self.__widget.set_size_request(-1, -1)
@@ -166,16 +164,13 @@ class TargetLabel(DisplayTarget):
 
     def __set_color(self, color):
 
+        # chop off alpha channel
+        if (color.startswith("#") and len(color) == 9): color = color[:7]
         self.__widget.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse(color))
 
 
     def __set_wrap(self, value):
 
-        #size, unit = value
-        #if (unit == Unit.UNIT_PERCENT):
-            #size = width * (size / 100.0)
-            #unit = Unit.UNIT_PX
-        #u = Unit.Unit(size, unit)
         width = self._get_parent().get_geometry()[2]
         if (width > Unit.ZERO): value.set_100_percent(width.as_px())
         size = value.as_px()
@@ -225,4 +220,6 @@ class TargetLabel(DisplayTarget):
 
         self.__set_wrap(value)
         self._setp(key, value)
+	self.__set_value(self.get_prop("value"))
         self.set_prop("value", self.get_prop("value"))
+
