@@ -1,10 +1,11 @@
-from DisplayContainer import DisplayContainer
 from GlassWindow import GlassWindow
+from config.StateSaver import DefaultStateSaver
 from utils.WindowSnapper import WindowSnapper
+import utils
 from main import ICON, UNSET_COORD
 
-import gtk
 import gobject
+import gtk
 
 
 # the available window flags
@@ -19,7 +20,7 @@ _WINDOW_FLAG_DECORATED = 1 << 4
 #
 # Class for display windows.
 #
-class Window(GlassWindow, DisplayContainer):
+class Window(GlassWindow):
 
     # mapping: str -> window flag
     __WINDOW_FLAGS = {"below"  : _WINDOW_FLAG_BELOW,
@@ -30,10 +31,7 @@ class Window(GlassWindow, DisplayContainer):
 
     __window_snapper = WindowSnapper()
 
-    def __init__(self):
-
-        # ID of the idle handler for initial placement of the window
-        self.__init_drag_handler = 0
+    def __init__(self, display):
 
         # window bbox as it was stored in the window snapper
         self.__window_bbox = (0, 0, 0, 0)
@@ -43,7 +41,6 @@ class Window(GlassWindow, DisplayContainer):
 
         # window position for detecting moves
         self.__window_pos = (UNSET_COORD, UNSET_COORD)
-        self.__is_placed = False
 
         # window size for detecting resizing
         self.__window_size = (0, 0)
@@ -61,13 +58,9 @@ class Window(GlassWindow, DisplayContainer):
         self.__drag_offset = (0, 0)
         self.__dont_snap = False
 
-        self.__reentrance_lock = False
-        self.__positioning_lock = False
-
         self.__shape = None
 
         GlassWindow.__init__(self, gtk.WINDOW_TOPLEVEL)
-        DisplayContainer.__init__(self)
 
         self.set_size_request(-1, -1)
         self.set_default_size(10, 10)
@@ -75,19 +68,68 @@ class Window(GlassWindow, DisplayContainer):
         # set the icon
         self.set_icon(gtk.gdk.pixbuf_new_from_file(ICON))
 
+        self.__display = display
+
+
+        display.show()
+        self.add(display)
+        display.add_observer(self.__on_observe_display)
+
         # set up event handlers
+        self.connect("delete-event", self.__on_close)
         self.connect("key-press-event", self.__on_key, 0)
         self.connect("key-release-event", self.__on_key, 1)
         self.connect("button-press-event", self.__on_button, 0)
-        self.connect("button-release-event", self.__on_button, 1)
+        self.connect("button-release-event", self.__on_button, True)
+        self.connect("motion-notify-event", self.__on_motion, False)
+        self.connect("leave-notify-event", self.__on_motion, True)
+        self.connect("scroll-event", self.__on_scroll)
         self.add_events(gtk.gdk.BUTTON_PRESS_MASK |
                         gtk.gdk.BUTTON_RELEASE_MASK |
-                        gtk.gdk.POINTER_MOTION_MASK |
-                        gtk.gdk.POINTER_MOTION_HINT_MASK)
+                        gtk.gdk.LEAVE_NOTIFY_MASK |
+                        gtk.gdk.POINTER_MOTION_MASK)
+                        # POINTER_MOTION_HINT_MASK caused problems...
+                        # gtk.gdk.POINTER_MOTION_HINT_MASK)
 
         self.realize()
-        self.set_window_flags("")
+        #self.set_window_flags("")
+        
 
+    #
+    # Observer handler for the display.
+    #
+    def __on_observe_display(self, src, cmd, *args):
+
+        if (cmd == src.OBS_GEOMETRY):
+            x, y, w, h = args
+            if (x == UNSET_COORD or y == UNSET_COORD):
+                # let the user place the window
+                self.__begin_dragging()
+            else:
+                self.set_position(x, y, True)
+                self.set_size(w, h)
+
+        elif (cmd == src.OBS_FLAGS):
+            flags = args[0]
+            utils.request_call(self.__set_window_flags, flags)
+
+        elif (cmd == src.OBS_ICON):
+            icon = args[0]
+            self.set_icon(icon)
+
+        elif (cmd == src.OBS_SHAPE):
+            shape = args[0]
+            self.__set_shape(shape)
+
+        elif (cmd == src.OBS_TITLE):
+            title = args[0]
+            self.set_title(title)
+
+        elif (cmd == src.OBS_CLOSED):
+            wx, wy, w, h = self.__window_bbox
+            self.__window_snapper.remove(wx, wy, w, h)
+            self.destroy()
+                    
 
 
     #
@@ -116,7 +158,7 @@ class Window(GlassWindow, DisplayContainer):
         assert(self.window)
 
         self.__is_dragging = True
-        self.update_observer(self.OBS_LOCK)
+        #self.update_observer(self.OBS_LOCK)
         x, y = self.get_pointer()
         w, h = self.__window_size
         w = max(5, w)
@@ -133,8 +175,8 @@ class Window(GlassWindow, DisplayContainer):
         # grab the window focus to ensure that keypresses work
         self.present()
 
-        gtk.timeout_add(50, self.__do_dragging)
-        gtk.timeout_add(50, self.__do_move_window)
+        gobject.timeout_add(50, self.__do_dragging)
+        gobject.timeout_add(50, self.__do_move_window)
 
         return False
 
@@ -147,21 +189,18 @@ class Window(GlassWindow, DisplayContainer):
 
         assert(self.window)
 
-        self.get_children()[0].window.shape_combine_mask(None, 0, 0)
+        self.get_children()[0].window.shape_combine_mask(self.__shape, 0, 0)
         self.__is_dragging = False
-        self.update_observer(self.OBS_UNLOCK)
+        #self.update_observer(self.OBS_UNLOCK)
         self.window.set_cursor(None)
 
         x, y = self.__window_pos
         w, h = self.__window_size
         if (not self.__managed_mode):
             self.__window_snapper.insert(x, y, w, h)
-        self.__set_placed()
-        self.__window_bbox = (x, y, w, h)
 
-        self.__reentrance_lock = True
-        self.update_observer(self.OBS_GEOMETRY)
-        self.__reentrance_lock = False
+        self.__window_bbox = (x, y, w, h)
+        self.__display.set_position(x, y)
 
         return False
 
@@ -204,7 +243,7 @@ class Window(GlassWindow, DisplayContainer):
     def __do_move_window(self):
 
         x, y = self.__window_pos
-        self.move(x, y)
+        utils.request_call(self.move, x, y)
 
         return self.__is_dragging
 
@@ -234,6 +273,26 @@ class Window(GlassWindow, DisplayContainer):
         else:
             self.__dont_snap = False
 
+        if (not self.__is_dragging):
+            px, py = self.get_pointer()
+            if (is_release):
+                self.__display.send_event(self.__display.EVENT_KEY_RELEASE,
+                                          key, px, py)
+            else:
+                self.__display.send_event(self.__display.EVENT_KEY_PRESS,
+                                          key, px, py)
+                                          
+
+
+    def __on_motion(self, src, event, is_leave):
+
+        if (is_leave):
+            utils.request_call(self.__display.send_event,
+                               self.__display.EVENT_LEAVE)
+        else:
+            utils.request_call(self.__display.send_event,
+                               self.__display.EVENT_MOTION, event.x, event.y)
+
 
 
     def __on_button(self, src, event, is_release):
@@ -247,42 +306,45 @@ class Window(GlassWindow, DisplayContainer):
         elif (is_release and self.__is_dragging):
             self.__end_dragging()
 
-
-
-    def close(self):
-
-        wx, wy, w, h = self.__window_bbox
-        self.__window_snapper.remove(wx, wy, w, h)
-        self.destroy()
-
-
-
-    def set_position(self, x, y):
-
-        gtk.idle_add(self.show)
-
-        # enter positioning mode if the values get unset
-        if (x == y == UNSET_COORD and not self.__is_dragging):
-            if (self.__managed_mode and not self.__is_placed):
-                # let the window manager place the window
-                self.__set_placed()
-
+        px, py = self.get_pointer()
+        if (is_release):
+            self.__display.send_event(self.__display.EVENT_RELEASE,
+                                      event.button, px, py)
+        else:
+            if (event.type == gtk.gdk._2BUTTON_PRESS):
+                counter = 2
             else:
-                # let the user place the window
-                self.__begin_dragging()
-                x, y = self.get_pointer()
-                self.__dig_a_hole(x, y)
-            return
+                counter = 1
+                
+            self.__display.send_event(self.__display.EVENT_PRESS,
+                                      event.button, px, py, counter)
 
-        # break cycles
-        if ((x, y) == self.__window_pos or self.__reentrance_lock
-            or self.__is_dragging): return
 
-        ox, oy = self.__window_pos
-        if ((x, y) != self.__window_pos):
-            self.__positioning_lock = True
-            gtk.timeout_add(0, self.move, x, y)
+    def __on_scroll(self, src, event):
+
+        px, py = self.get_pointer()
+        self.__display.send_event(self.__display.EVENT_SCROLL, event.direction,
+                                  px, py)
+
+
+    def __on_close(self, src, *args):
+
+        self.__display.close()
+        return True
+
+
+
+
+    #
+    # Sets the position of the window. If dont_set is True, the display will not
+    # be notified about the movement.
+    #
+    def set_position(self, x, y, dont_set = False):
+
+        if (self.__window_pos != (x, y) and not self.__is_dragging):
+            utils.request_call(self.move, x, y)
             self.__window_pos = (x, y)
+            if (not dont_set): self.__display.set_position(x, y)
 
             bx, by, bw, bh = self.__window_bbox
             if (not self.__managed_mode):
@@ -290,47 +352,39 @@ class Window(GlassWindow, DisplayContainer):
             w, h = self.__window_size
             if (not self.__managed_mode):
                 self.__window_snapper.insert(x, y, w, h)
-            self.__set_placed()
+
             self.__window_bbox = (x, y, w, h)
-            self.update_observer(self.OBS_GEOMETRY)
-
-
-    def __set_placed(self):
-
-        self.__is_placed = True
-        if (self.__shape): self.set_shape(self.__shape)
-
-
-    def is_placed(self):
-
-        return self.__is_placed
+            gobject.idle_add(self.show)
 
 
 
+
+    #
+    # Sets the size of the window.
+    # 
     def set_size(self, width, height):
 
         if ((width, height) != self.__window_size):
             bx, by, bw, bh = self.__window_bbox
+
             if (not self.__managed_mode):
                 self.__window_snapper.remove(bx, by, bw, bh)
+
             self.__window_size = (width, height)
             self.resize(width, height)
+
             if (not self.__managed_mode):
                 self.__window_snapper.insert(bx, by, width, height)
+
             self.__window_bbox = (bx, by, width, height)
-            self.update_observer(self.OBS_GEOMETRY)
 
 
 
-    def get_geometry(self):
 
-        x, y = self.__window_pos
-        w, h = self.__window_size
-        return (x, y, w, h)
-
-
-
-    def set_window_flags(self, value):
+    #
+    # Sets the window flags.
+    #
+    def __set_window_flags(self, value):
 
         is_managed = self.__managed_mode
         flags = _WINDOW_FLAG_NONE
@@ -340,13 +394,10 @@ class Window(GlassWindow, DisplayContainer):
             try:
                 flags |= self.__WINDOW_FLAGS[p]
             except KeyError:
-                import sys
-                print >> sys.stderr, "%s is not a valid window flag" % (p,)
+                log("Warning: %s is not a valid window flag." % (p,))
 
-        if (not flags & _WINDOW_FLAG_ABOVE):
-            self._set_flag_below(flags & _WINDOW_FLAG_BELOW)
-        if (not flags & _WINDOW_FLAG_BELOW):
-            self._set_flag_above(flags & _WINDOW_FLAG_ABOVE)
+        self._set_flag_below(flags & _WINDOW_FLAG_BELOW)
+        self._set_flag_above(flags & _WINDOW_FLAG_ABOVE)
         self._set_flag_sticky(flags & _WINDOW_FLAG_STICKY)
         self._set_flag_managed(flags & _WINDOW_FLAG_MANAGED)
         self._set_flag_decorated(flags & _WINDOW_FLAG_DECORATED)
@@ -366,21 +417,11 @@ class Window(GlassWindow, DisplayContainer):
 
 
 
-    def set_title(self, value):
+    #
+    # Sets the window's shape.
+    #
+    def __set_shape(self, mask):
 
-        GlassWindow.set_title(self, value)
-
-
-
-    def set_icon(self, pixbuf):
-
-        GlassWindow.set_icon(self, pixbuf)
-
-
-
-    def set_shape(self, mask):
-
-        if (self.__window_pos == (UNSET_COORD, UNSET_COORD)):
-            self.__shape = mask
-        else:
+        self.__shape = mask
+        if (not self.__is_dragging):
             self.shape_combine_mask(mask, 0, 0)
