@@ -1,20 +1,15 @@
-from config.ConfigSettings import ConfigSettings
 from ControlFactory import ControlFactory
 from utils.TargetSettings import TargetSettings
 from display.Display import Display
-from utils.TypeConverter import TypeConverter
 from utils.datatypes import *
 from utils import dialog
+from utils import vfs
 
 from xml import sax
 
 import os
 import sys
 
-try:
-    import gnomevfs
-except ImportError:
-    import gnome.vfs as gnomevfs
 
 _STATE_NONE = -1
 _STATE_DISPLAY = 0
@@ -48,46 +43,61 @@ class _DisplayFactory(sax.handler.ContentHandler):
         # a list of the sensors for the display
         self.__sensors = []
 
-        # the config settings object of the display
-        self.__config_settings = None
+        # the config items of the display
+        self.__config_items = []
 
         self.__path = ""
         self.__scripts = []
         self.__script = ""
         self.__id = ""
         self.__control_factory = ControlFactory()
-        self.__type_converter = TypeConverter()
+        self.__prefs_callback = None
+        self.__metadata = {}
 
         sax.handler.ContentHandler.__init__(self)
-        self.__type_converter.add_type("module", TYPE_LIST)
 
 
 
+    def __reset(self):
+
+        self.__scripts = []
+        self.__config_items = []
+        self.__children_stack = [[]]
+        self.__nesting_stack = []
+        self.__sensors = []
+        self.__prefs_callback = None
+        self.__metadata = {}
+
+
     #
     # Parses the given XML data and returns a new Display object.
     #
-    def create_display(self, ident, data, rep):
+    def create_display(self, ident, data, dspfile):
 
-        dsp = Display(ident, rep)
+        dsp = Display(ident, dspfile)
 
-        self.__config_settings = ConfigSettings()
-        self.__scripts = []
+        self.__reset()
         self.__id = ident
-        self.__children_stack = [[]]
-        self.__nesting_stack = []
-        self.__sensors = []
-        self.__path = os.path.dirname(rep)
+        self.__path = os.path.dirname(dspfile)
+        self.__filename = dspfile
 
         # parse display data
         try:
             sax.parseString(data, self)
+
         except sax._exceptions.SAXParseException, sax_exc:
-            print >> sys.stderr, "Parse Error: %s" % (sax_exc,)
+            from utils.ErrorFormatter import ErrorFormatter
+            details = ErrorFormatter().format(sys.exc_info(), code = data,
+                                              lineno = sax_exc.getLineNumber())
+            dialog.warning(_("XML parse error"),
+                           _("This .display file has invalid XML syntax."),
+                           details)
+            log("Parse Error:\n%s" % (details,))
             # abort if a parse error occured
             return
         except StandardError:
-            import traceback
-	    traceback.print_exc()
+            from utils.ErrorFormatter import ErrorFormatter
+            print ErrorFormatter().format(sys.exc_info())
             return
 
         # add the sensor controls
@@ -97,30 +107,36 @@ class _DisplayFactory(sax.handler.ContentHandler):
         # add the children and configure
         try:
             childtype, settings, children = self.__children_stack.pop()[0]
+            #dsp.new_child(childtype, settings, children)
+            dsp.set_content(childtype, settings, children)
 
-            dsp.new_child(childtype, settings, children)
-            for s in self.__scripts:
-                dsp.execute_script(s)
+        except:
+            from utils.ErrorFormatter import ErrorFormatter
+            print ErrorFormatter().format(sys.exc_info())
 
-        except StandardError, exc:
-            # a stack trace could be useful for debugging
-            import traceback
-	    traceback.print_exc()
-
-            # make the corrupt display visible so that the user can remove it
-            print >> sys.stderr, "Error while configuring display: %s" % (exc,)
-            dialog.warning(_("Error while configuring display"),
-                           _("A display could not be configured properly. "
+            dialog.warning(_("Error while configuring desklet"),
+                           _("A desklet could not be configured properly. "
                              "It will most likely be broken and you should "
                              "consider removing it."))
             return
 
-        dsp.set_configurator(self.__config_settings)
+        preview = self.__metadata.get("preview", "")
+        name = self.__metadata.get("name", "")
+        version = self.__metadata.get("version", "")
+        author = self.__metadata.get("author", "")
+        dsp.set_metadata(preview = preview,
+                         name = name,
+                         version = version,
+                         author = author)
+        dsp.build_configurator(self.__config_items)
 
-        self.__children_stack = [[]]
-        self.__nesting_stack = []
-        self.__sensors = []
-        self.__scripts = []
+
+        for script, filename in self.__scripts:
+            dsp.add_scriptlet(script, filename)
+
+        # finished. initialize and return
+        #dsp.initialize()
+        self.__reset()
 
         return dsp
 
@@ -141,21 +157,23 @@ class _DisplayFactory(sax.handler.ContentHandler):
 
     def startElement(self, name, attrs):
 
+        attrs_dict = {}
+        attrs_dict.update(attrs)
+
         if (name == "display"):
             self.__state.append(_STATE_DISPLAY)
 
         elif (name == "prefs"):
+            # support a global prefs callback
+            self.__prefs_callback = attrs.get("callback", None)
             self.__state.append(_STATE_PREFS)
 
         elif (name == "meta"):
             self.__state.append(_STATE_META)
 
         elif (name == "sensor"):
-            # the <sensor> tag is deprecated
-            import warnings
-            warnings.warn("Using sensors is deprecated now. Please consider "
-                          "using controls and inline scripts now.",
-                          DeprecationWarning)
+            log("Deprecation: Sensors are deprecated since v0.30.\n"
+                "Please consider using controls and inline scripts.")
             self.__state.append(_STATE_SENSOR)
 
         elif (name == "control"):
@@ -169,7 +187,7 @@ class _DisplayFactory(sax.handler.ContentHandler):
             self.__children_stack.append([])
 
         # remember everything for later
-        self.__nesting_stack.append((name, attrs))
+        self.__nesting_stack.append((name, attrs_dict))
 
 
 
@@ -180,8 +198,7 @@ class _DisplayFactory(sax.handler.ContentHandler):
         if (not name == oname):
             # nesting errors in XML are detected by the SAX parser; if we
             # get here, it means our parser is buggy, not the XML input
-            print >> sys.stderr, "Nesting error: expected %s, got %s." % \
-	                                                         (oname, name)
+            log("Nesting error: expected %s, got %s." % (oname, name))
             return
 
 
@@ -196,7 +213,7 @@ class _DisplayFactory(sax.handler.ContentHandler):
             self.__children_stack[-1].append((name, settings, children))
 
         elif (state == _STATE_META):
-            pass
+            self.__metadata = attrs
 
         elif (state == _STATE_SCRIPT):
             # Some distros ship a broken SAX parser and
@@ -204,23 +221,29 @@ class _DisplayFactory(sax.handler.ContentHandler):
             # So we have to use has_key()
             # if ((name == "script") and ("uri" in attrs)):
             if (name == "script" and attrs.has_key("uri")):
+                path = attrs["uri"]
+                filename = os.path.join(self.__path, path)
                 try:
-                    path = attrs["uri"]
-                    path = os.path.join(self.__path, path)
-                    self.__script += "\n" + gnomevfs.read_entire_file(path)
-                except StandardError:
-                    import traceback
-		    traceback.print_exc()
+                    self.__script += "\n" + vfs.read_entire_file(filename)
+                except:
+                    log("File doesn't exist or couldn't be loaded: %s"
+                        % (path,))
+                    dialog.warning(_("File doesn't exist or couldn't be "
+                                     "loaded"),
+                                   _("A file which contains a script isn't "
+                                     "available for further execution."))
+            else:
+                filename = self.__filename
 
             # collect script by script to be able to execute them one by one
             # later; we would get problems with different indentation otherwise
-            self.__scripts.append(self.__script)
+            self.__scripts.append((self.__script, filename))
 
         # deprecated sensor stuff
         elif (state == _STATE_SENSOR):
+            from utils import typeconverter
             ident = attrs["id"]
-            moduledata = self.__type_converter.str2type("module",
-                                                        attrs["module"])
+            moduledata = typeconverter.str2type(TYPE_LIST, attrs["module"])
             module = moduledata[0]
             args = moduledata[1:]
             sensorctrl = self.__control_factory.get_control(_SENSOR_CONTROL)
@@ -236,14 +259,27 @@ class _DisplayFactory(sax.handler.ContentHandler):
             ident = attrs["id"]
             interface = attrs["interface"]
             script = "%s = get_control('%s')" % (ident, interface)
-            self.__scripts.append(script)
+            self.__scripts.append((script, self.__filename))
 
         elif (state == _STATE_PREFS):
             if (name == "prefs"):
-                callback = attrs.get("callback", None)
-                if (callback): self.__config_settings.set_callback(callback)
+                pass
             else:
-                self.__config_settings.add_item(name, attrs)
+                if (name in ("boolean", "color", "enum", "float", "font",
+                             "integer", "string", "uri")
+                    and not attrs.has_key("bind")):
+                    dialog.warning(_("&lt;%s&gt; needs a bind attribute") % \
+                                                                      (name,),
+                                   _("The &lt;prefs&gt; section of this "
+                                     "desklet file is broken."))
+                    raise AttributeError("<%s> needs a bind attribute" % \
+                                                                      (name,))
+
+                # use global prefs callback if no callback was given
+                if (self.__prefs_callback and not attrs.has_key("callback")):
+                    attrs["callback"] = self.__prefs_callback
+
+                self.__config_items.append((name, attrs))
 
 
         if (name in ("display", "sensor", "control", "meta",
