@@ -1,3 +1,5 @@
+from config import settings
+from config.DaemonConfigger import DaemonConfigger
 from display.Display import Display
 from display.Window import Window
 from display.Plug import Plug
@@ -6,19 +8,14 @@ from main import COPYRIGHT, HOME, NAME, USERHOME, VERSION, POSITIONSFILE
 from main.AboutDialog import AboutDialog
 from main.remotecommands import *
 from main.RemoteSocket import RemoteSocket
-from main.TrayIcon import TrayIcon
 from utils import dialog
+from utils import vfs
 
+import gobject
 import gtk
-import shutil
 import os
 import sys
 
-try:
-    import gnomevfs
-except ImportError:
-    import gnome.vfs as gnomevfs
-
 
 # This class starts, restarts and kills Displays
 class Starter:
@@ -32,23 +29,22 @@ class Starter:
         # the communication socket
         self.__socket = RemoteSocket()
 
+        # the factory which reads the display's XML
+        self.__factory = DisplayFactory()
+
         # the About dialog
         self.__about_dialog = AboutDialog(os.path.join(HOME, "data"))
 
+        # the configuration dialog
+        self.__configger = DaemonConfigger()
+
         # the command which gets called when the user removes a display
         self.__remove_command = ""
 
-        # the restart command which gets called when the session restarts
-        self.__startup_command = ""
-
         # the set of open displays as a hashtable "id -> display"
         self.__open_displays = {}
         # the paths of the display files "id -> path"
         self.__display_paths = {}
-        # the saved positions of the displays "id -> (x, y)"
-        self.__positions = {}
-
-        self.__factory = DisplayFactory()
 
         # set the message handlers for the socket
         self.__socket.add_message_handler(COMMAND_OPEN_DISPLAY,
@@ -71,33 +67,33 @@ class Starter:
                                           self.__handle_version)
         self.__socket.add_message_handler(COMMAND_ABOUT,
                                           self.__handle_about)
+        self.__socket.add_message_handler(COMMAND_CONFIGURE,
+                                          self.__handle_configger)
         self.__socket.add_message_handler(COMMAND_SET_REMOVE_COMMAND,
                                           self.__handle_set_remove_command)
-        self.__socket.add_message_handler(COMMAND_SET_STARTUP_COMMAND,
-                                          self.__handle_set_startup_command)
 
-        self.__load_positions()
 
         # socket ready, start handling requests
         self.__socket.start()
 
         # setup a nice systray icon
-        trayicon = TrayIcon()
-        trayicon.set_menu([(gtk.STOCK_PROPERTIES, _("_Manage desklets"),
-                            self.__handle_manage),
-                           (None, _("_View log"),
-                           # (gtk.STOCK_EDIT, _("_View log"),
-                            self.__handle_show_log),
-                           ("gnome-stock-about", _("_About..."), self.__handle_about_dialog),
-                           (),
-                           (gtk.STOCK_QUIT, _("_Stop daemon"),
-                            self.__handle_shutdown)])
+        if (not HAVE_WIN32 and settings.show_tray_icon):
+            from main.TrayIcon import TrayIcon
+            trayicon = TrayIcon()
+            trayicon.set_menu([(None, _("_Manage desklets"),
+                                self.__handle_manage),
+                               (),
+                               (gtk.STOCK_PROPERTIES, _("_Configuration"),
+                                 self.__handle_configger),
+                               (None, _("_View log"),
+                                self.__handle_show_log),
+                                (),
+                               (None, _("_About..."),
+                                self.__handle_about_dialog),
+                               (),
+                               (gtk.STOCK_QUIT, _("_Stop daemon"),
+                                self.__handle_shutdown)])
 
-        def save_pos():
-            self.__save_positions()
-            return True
-
-        gtk.timeout_add(3000, save_pos)
 
 
 
@@ -116,11 +112,10 @@ class Starter:
             xcoord, ycoord = src.get_geometry()[0:2]
             xcoord = xcoord.as_px()
             ycoord = ycoord.as_px()
-            self.__positions[ident] = (xcoord, ycoord)
             path = self.__display_paths[ident]
             self.__remove_display(ident)
-            gtk.timeout_add(250, self.__add_display, ident, path,
-                            self.__DISPLAY_TYPE_WINDOW)
+            gobject.timeout_add(250, self.__add_display, ident, path,
+                                self.__DISPLAY_TYPE_WINDOW)
             del src
 
 
@@ -134,64 +129,60 @@ class Starter:
         container = None
 
         if (ident not in self.__open_displays):
-            dsp = self.__create_display(ident, path)
+            try:
+                dsp = self.__create_display(ident, path)
+            except (IOError, SyntaxError), exc:
+                log("Warning: Couldn't add desklet \"%s\".\n%s"  % (path, exc))
+                return
 
-            if (dsp):
-                self.__open_displays[ident] = dsp
-                self.__display_paths[ident] = path
+            self.__open_displays[ident] = dsp
+            self.__display_paths[ident] = path
 
-                print "Info: Adding [%s] with ID '%s' to the list of " \
-                      "displays." % (path, ident)
-                dsp.add_observer(self.__on_display_action)
+            log("Adding \"%s\" with ID \"%s\" to the desklet list."
+                % (path, ident))
+            dsp.add_observer(self.__on_display_action)
 
-                if (displaytype == self.__DISPLAY_TYPE_WINDOW):
-                    container = Window()
-                    try:
-                        xcoord, ycoord = self.__positions[ident]
-                        dsp.set_position(xcoord, ycoord)
-                    except KeyError:
-                        dsp.set_position()
+            if (displaytype == self.__DISPLAY_TYPE_WINDOW):
+                container = Window(dsp)
 
-                    dsp.set_container(container)
+            elif (displaytype == self.__DISPLAY_TYPE_PLUG):
+                container = Plug()
+                dsp.set_container(container)
+                container.show()
 
-                elif (displaytype == self.__DISPLAY_TYPE_PLUG):
-                    container = Plug()
-                    dsp.set_container(container)
-                    container.show()
+            dsp.initialize()
 
         return container
 
 
 
     #
-    # Creates and returns a new display from the given data, or None in case
-    # of an error.
+    # Creates and returns a new display from the given data.
     #
     def __create_display(self, ident, path):
 
-        try:
-            if (not gnomevfs.exists(path)):
-                raise IOError(_("File does not exist."))
-            data = gnomevfs.read_entire_file(path)
+        if (not vfs.exists(path)):
+            raise IOError("Warning: The file \"%s\" doesn't exist." % (path,))
 
-        except StandardError:
-            import traceback
-            traceback.print_exc()
-            print >> sys.stderr, "Could not open display file [%s]." % (path,)
-            dialog.warning(_("Could not open display file '%(path)s'") % vars(),
-                           _("The display file could not be opened because "
-                             "the file was not readable."), False)
+        try:
+            data = vfs.read_entire_file(path)
+
+        except:
+            log("Could not open desklet file \"%s\"." % (path,))
+            dialog.warning(_("Could not open desklet file \"%s\"") % (path,),
+                           _("The desklet file could not be opened because "
+                             "the file was not readable."))
             self.__close_display(ident)
             return
 
-
         display = self.__factory.create_display(ident, data, path)
 
         if (not display):
-            dialog.warning(_("Invalid display file '%(path)s'") % vars(),
-                           _("The display file contains invalid data and "
-                             "could not be loaded."), False)
+            dialog.warning(_("Invalid desklet file \"%s\"") % (path,),
+                           _("The desklet file contains invalid data and "
+                             "could not be loaded."))
             self.__close_display(ident)
+            raise SyntaxError("Warning: Invalid desklet file \"%s\"" % (path,))
 
         return display
 
@@ -202,20 +193,27 @@ class Starter:
     #
     def __remove_display(self, ident):
 
-        if (ident in self.__open_displays):
-            # better use python2.3 dict.pop
+        try:
             display = self.__open_displays[ident]
-            xcoord, ycoord = display.get_geometry()[0:2]
-            xcoord = xcoord.as_px()
-            ycoord = ycoord.as_px()
-            self.__positions[ident] = (xcoord, ycoord)
-            if display:
-                print "Info: Removing [%s] with ID '%s' from the list of " \
-                      "displays." % (display.get_path(), ident)
-                display.remove_display()
+        except KeyError, exc:
+            log("Warning: Couldn't remove desklet with ID \"%s\".\n%s" \
+                % (ident, exc))
+            return
+
+        log("Removing \"%s\" with ID \"%s\" from the desklet list."
+            % (self.__display_paths[ident], ident))
+        display.remove_display()
+
+        try:
             del self.__open_displays[ident]
             del self.__display_paths[ident]
+        except StandardError:
+            pass
 
+        import gc
+        # gc.set_debug(gc.DEBUG_STATS)
+        gc.collect()
+        #print len(gc.garbage)
 
 
     #
@@ -226,67 +224,11 @@ class Starter:
         if (ident in self.__open_displays):
             display = self.__open_displays[ident]
             display.purge_display()
-            
+
         self.__remove_display(ident)
         if (self.__remove_command):
             os.system("%s %s &" % (self.__remove_command, ident))
 
-        try:
-            del self.__positions[ident]
-        except StandardError:
-            pass
-
-
-
-    #
-    # Loads the display positions from file.
-    #
-    def __load_positions(self):
-
-        try:
-            lines = open(POSITIONSFILE, "r").readlines()
-        except StandardError:
-            # no positions
-            return
-
-        for line in lines:
-            if (not line.strip()):
-                continue
-            ident, xcoord, ycoord = line.split()
-            self.__positions[ident] = (int(xcoord), int(ycoord))
-
-
-
-    #
-    # Saves the current display positions to a file.
-    #
-    def __save_positions(self):
-
-        changed = False
-
-        for ident, display in self.__open_displays.items():
-            xcoord, ycoord, = display.get_geometry()[0:2]
-            xcoord = xcoord.as_px()
-            ycoord = ycoord.as_px()
-            if (self.__positions.get(ident) != (xcoord, ycoord)):
-                changed = True
-            self.__positions[ident] = (xcoord, ycoord)
-
-        if (not changed):
-            return
-
-        data = ""
-        for ident, pos in self.__positions.items():
-            xcoord, ycoord = pos
-            data += "%s %d %d\n" % (ident, xcoord, ycoord)
-        try:
-            open(POSITIONSFILE, "w").write(data)
-        except IOError:
-            import traceback
-            traceback.print_exc()
-            print >> sys.stderr, "Could not write the position file [%s]." % \
-                                                              (POSITIONSFILE,)
-
 
 
     def __handle_open_display(self, path):
@@ -302,8 +244,6 @@ class Starter:
 
         self.__add_display(ident, path, self.__DISPLAY_TYPE_WINDOW)
 
-        return (None,)
-
 
 
     def __handle_open_plug(self, path):
@@ -321,8 +261,6 @@ class Starter:
 
         self.__remove_display(ident)
 
-        return (None,)
-
 
 
     def __handle_version(self):
@@ -341,15 +279,15 @@ class Starter:
 
     def __handle_shutdown(self, *args):
 
-        self.__save_positions()
+        #self.__save_positions()
 
         for ident, display in self.__open_displays.items():
             try:
                 display.remove_display()
-            except StandardError:
-                pass
+            # what kind of exception are we expecting here?
+            except:
+                log("Could not remove desklet \"%s\"!" % (display,))
         gtk.main_quit()
-        return (None,)
 
 
 
@@ -359,6 +297,11 @@ class Starter:
         os.system(cmd + " &")
 
 
+    def __handle_configger(self, *args):
+
+        self.__configger.show()
+
+
 
     def __handle_about_dialog(self, *args):
 
@@ -398,35 +341,4 @@ class Starter:
         if (not self.__remove_command):
             self.__remove_command = command
 
-        return (None,)
 
-
-
-    def __handle_set_startup_command(self, command):
-
-        # only allow setting the startup command once for security reasons
-        if (not self.__startup_command):
-            self.__startup_command = command
-
-            # connect to gnome-session
-            try:
-                import gnome.ui
-                # does the required method exist?
-                gnome.ui.Client.set_restart_command
-                client = gnome.ui.master_client()
-                if (client):
-                    client.set_restart_style(gnome.ui.RESTART_IF_RUNNING)
-                    cmd = command.split()
-                    try:
-                        client.set_restart_command(*cmd)
-                    except StandardError:
-                        client.set_restart_command(len(cmd), cmd)
-            except StandardError:
-                import traceback
-                traceback.print_exc()
-                print >> sys.stderr, "Could not connect to gnome-session."
-                print _("Could not connect to gnome-session. "
-                        "GNOME session management "
-                        "will not be available for this application.")
-
-        return (None,)
