@@ -1,17 +1,48 @@
 import os
 import sys
+import NullObject
+import __builtin__
+
+
+HAVE_GNOME = True
+
+try:
+    import gnomevfs
+except ImportError:
+    try:
+        import gnome.vfs
+    except ImportError:
+        HAVE_GNOME = False
+
+if (sys.platform == "win32"):
+    HAVE_WIN32 = True
+else:
+    # no, we're lucky
+    HAVE_WIN32 = False
+
+__builtin__.HAVE_GNOME = HAVE_GNOME
+__builtin__.HAVE_WIN32 = HAVE_WIN32
 
 # gDesklets home paths
 HOME = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))
-USERHOME = os.path.join(os.path.expanduser("~"), ".gdesklets")
+if (HAVE_WIN32):
+    USERHOME = os.path.join(os.environ["APPDATA"], "gDesklets")
+else:
+    USERHOME = os.path.join(os.path.expanduser("~"), ".gdesklets")
 OLDHOME = os.path.join(HOME, os.pardir, os.pardir, "share", "gdesklets")
 
+
 # we need the DISPLAY variable
-try:
-    DISPLAY = os.environ["DISPLAY"]
-except KeyError:
-    print "The DISPLAY variable is NOT set, which means, that X isn't running!"
-    sys.exit(1)
+if (HAVE_WIN32):
+    DISPLAY = "win32"
+else:
+    try:
+        DISPLAY = os.environ["DISPLAY"]
+    except KeyError:
+        print "The DISPLAY variable is NOT set, which means, " \
+              "that X isn't running!"
+        sys.exit(1)
+
 
 # the path of the log file
 LOGFILE = os.path.join(USERHOME, "gdesklets" + DISPLAY + ".log")
@@ -26,17 +57,17 @@ PURGE_KEY = "_paths_to_purge_"
 UNSET_COORD = -1000
 
 # paths where the display files are located
-DISPLAYPATHS = [os.path.join(USERHOME, "Displays"),
+DISPLAYPATHS = (os.path.join(USERHOME, "Displays"),
                 os.path.join(HOME, "Displays"),
-                os.path.join(OLDHOME, "Displays")]
+                os.path.join(OLDHOME, "Displays"))
 # paths where gDesklets looks for sensors and displays
-SENSORPATHS = [os.path.join(USERHOME, "Sensors"),
+SENSORPATHS = (os.path.join(USERHOME, "Sensors"),
                os.path.join(HOME, "Sensors"),
-               os.path.join(OLDHOME, "Sensors")]
+               os.path.join(OLDHOME, "Sensors"))
 # paths where gDesklets looks for controls
-CONTROLPATHS = [os.path.join(USERHOME, "Controls"),
+CONTROLPATHS = (os.path.join(USERHOME, "Controls"),
                 os.path.join(HOME, "Controls"),
-                os.path.join(OLDHOME, "Controls")]
+                os.path.join(OLDHOME, "Controls"))
 # path of the communication sockets
 SOCKET_PATH = os.path.join(USERHOME, "sockets")
 
@@ -48,7 +79,7 @@ _ = Translator("gdesklets")
 
 
 NAME = "gDesklets"
-VERSION = "0.33.1"
+VERSION = "0.34.3"
 COPYRIGHT = u"Copyright \xa9 2003 - 2005 The gDesklets Team"
 DESCRIPTION = _("A desktop applet system for GNOME")
 AUTHORS = ( "Martin Grimme <martin@pycage.de>",
@@ -78,57 +109,28 @@ CREDITS = (
            ("Johannes Rebhan",)),
 
            (_("Website:"),
-           ("Luke Stroven",))
+           ("Luke Stroven",)),
+
+           (_("Thank you:"),
+           ("VidaLinux for sponsoring",
+            "the http://www.gdesklets.org domain!",))
           )
 
 
-# create the gDesklets home dir
-try:
-    if (not os.path.exists(USERHOME)):
-        os.makedirs(USERHOME)
-except StandardError:
-    pass
-
-
 def init():
 
     """ gDesklets init process """
-
-    try:
-        import gnome.ui
-    except ImportError:
-        print "The GNOME python bindings aren't (correctly) installed!"
-	print "Install this package, exiting!"
-        sys.exit(1)
-        
-    try:
-        import gtk
-    except ImportError:
-        print "The gtk+ python bindings aren't (correctly) installed!"
-	print "Install this package, exiting!"
-
     import signal
+    import gtk
 
     # prepare GTK for threads
     try:
         gtk.threads_init()
     except:
         print "No threading support available in python."
-	print "Compile python with --enable-threads. Exiting!"
+        print "Compile python with --enable-threads. Exiting!"
         sys.exit(1)
 
-    # init gnome
-    chldhandler = signal.getsignal(signal.SIGCHLD)
-    gnome.program_init(NAME, VERSION)
-
-    # gnome.program_init has the bad new habit to modify the locale settings
-    # (bug #135055); repair it
-    import locale
-    locale.setlocale(locale.LC_NUMERIC, "C")
-
-    # gnome.program_init < 1.99.18 breaks the SIGCHLD handler; gotta repair it
-    signal.signal(signal.SIGCHLD, chldhandler)
-
     # install signal handler to quit on Ctrl-C
     signal.signal(signal.SIGINT, gtk.main_quit)
 
