@@ -5,11 +5,8 @@
 #        [app:gDesklets] core, so you can fully concentrate on the sensor
 #        itself.
 
-import NullObject
-
 from utils.Observable import Observable
 from utils.TargetSettings import TargetSettings
-from utils.TypeConverter import TypeConverter
 from utils import i18n
 from utils import MAIN_THREAD
 from utils import run_in_main_thread
@@ -18,14 +15,13 @@ from SensorConfigurator import SensorConfigurator
 from main import _
 from main import PURGE_KEY
 
+import gobject
 import gtk
 import thread
-import threading
 import time
 import os
 
 
-
 #
 #:class Sensor |
 # Abstract base class for all sensors. Every sensor has to inherit from this
@@ -62,8 +58,9 @@ class Sensor(Observable):
         # when stopping, the flag is set to True, then the callback is called
         self.__stop_callback = Null
 
-        # converter for data types
-        self.__type_converter = TypeConverter()
+        # table: key -> datatype
+        self.__datatypes = {}
+
         self.__defaults = {}
 
 
@@ -128,10 +125,9 @@ class Sensor(Observable):
         assert(self.__config_manager)
 
         if (not self.__stop_flag):
-            value = self.__type_converter.type2str(key, value)
-            #self.__config_lock.acquire()
-            #self.__to_set.append((key, value))
-            #self.__config_lock.release()
+            from utils import typeconverter
+            dtype = self.__datatypes[key]
+            value = typeconverter.type2str(dtype, value)
             run_in_main_thread(self.__config_manager.set,
                                self.__config_id, key, value)
 
@@ -150,9 +146,13 @@ class Sensor(Observable):
         value = run_in_main_thread(self.__config_manager.get,
                                    self.__config_id, key)
 
-        if (value == ""): value = self.__defaults.get(key, "")
+        if (value == self.__config_manager.UNDEF):
+            value = self.__defaults.get(key, "")
+
         if (type(value) == str):
-            value = self.__type_converter.str2type(key, value)
+            from utils import typeconverter
+            dtype = self.__datatypes[key]
+            value = typeconverter.str2type(dtype, value)
 
         return value
 
@@ -185,7 +185,7 @@ class Sensor(Observable):
     #
     def _set_config_type(self, key, dtype, default):
 
-        self.__type_converter.add_type(key, dtype)
+        self.__datatypes[key] = dtype
         self.__defaults[key] = default
 
 
@@ -255,7 +255,7 @@ class Sensor(Observable):
             else:
                 return False
 
-        return gtk.timeout_add(interval, __wrap, self, callback, args)
+        return gobject.timeout_add(interval, __wrap, self, callback, args)
 
 
 
@@ -274,7 +274,7 @@ class Sensor(Observable):
             thread.start_new_thread(threadfunction, args)
             return False
 
-        gtk.idle_add(run_thread, threadfunction, args)
+        gobject.idle_add(run_thread, threadfunction, args)
 
 
 
@@ -419,31 +419,6 @@ class Sensor(Observable):
 
 
 
-    def __on_about(self, src):
-
-        from main import NAME, VERSION, COPYRIGHT, DESCRIPTION
-        from main import AUTHORS, DOCUMENTERS, ICON
-
-        import gnome.ui
-
-        tcredits = _("translator_credits")
-        if (tcredits == "translator_credits"): tcredits = ""
-        pixbuf = gtk.gdk.pixbuf_new_from_file(ICON)
-        about = gnome.ui.About(NAME, VERSION, COPYRIGHT, DESCRIPTION,
-                               AUTHORS, DOCUMENTERS, tcredits,
-                               pixbuf)
-        # set the icon
-        about.set_icon(gtk.gdk.pixbuf_new_from_file(ICON))
-
-        about.show()
-
-
-
-    def __on_quit(self, src):
-
-        gtk.main_quit()
-
-
     # some required aliases for older sensors to work
     _get_id = get_id
     _get_path = get_path
@@ -458,3 +433,4 @@ class Sensor(Observable):
     _get_config = get_config
     _set_paths_to_purge = set_paths_to_purge
     _open_menu = open_menu
+
