@@ -1,12 +1,9 @@
 import gobject
 import gtk
-import tiling
 import sys
+import vfs
+import utils
 
-try:
-    import gnomevfs
-except ImportError:
-    import gnome.vfs as gnomevfs
 
 #
 # Image widget that supports tiling.
@@ -52,15 +49,18 @@ class TilingImage(gtk.Image):
     #
     def set_from_file(self, uri):
 
-        if (gnomevfs.exists(uri)):
-            data = gnomevfs.read_entire_file(uri)
+        if (vfs.exists(uri)):
+            try:
+                data = vfs.read_entire_file(uri)
+            except:
+                return
             loader = gtk.gdk.PixbufLoader()
             try:
                 loader.write(data, len(data))
-                loader.close()
             except:
-                print "Invalid file format"
-                loader.close()
+                log("Invalid image format.")
+                return
+            loader.close()
             self.__pixbuf = loader.get_pixbuf()
             w, h = self.__size
             self.__is_invalidated = True
@@ -85,4 +85,31 @@ class TilingImage(gtk.Image):
         width = max(width, 1)
         height = max(height, 1)
 
-        tiling.tile_on_image (self, self.__pixbuf, width, height)
+        if (HAVE_WIN32):
+            self.tile_on_image_win32(self.__pixbuf, width, height)
+        else:
+            import tiling
+            tiling.tile_on_image (self, self.__pixbuf, width, height)
+
+
+    def tile_on_image_win32(self, pixbuf, width, height):
+
+        w = pixbuf.get_width()
+        h = pixbuf.get_height()
+
+        bg = gtk.gdk.Pixbuf(0, 1, 8, width, height)
+        for x in xrange(0, width, w):
+            for y in xrange(0, height, h):
+                pw = min(w, width - x)
+                ph = min(h, height - y)
+                pixbuf.copy_area(0, 0, pw, ph, bg, x, y)
+
+        self.set_from_pixbuf(bg)
+
+        cache_width = int(width / w) * w
+        cache_height = int(height / h) * h
+        if (cache_width > 0 and cache_height > 0):
+            cache = gtk.gdk.Pixbuf(0, 1, 8, cache_width, cache_height)
+            bg.copy_area(0, 0, cache_width, cache_height, cache, 0, 0)
+            self.__pixbuf = cache
+
