@@ -4,8 +4,8 @@ import traceback
 
 from cStringIO import StringIO
 from HIGDialog import HIGDialog
+from LogView import LogView
 
-# Functions for creating every kind of dialog. They are for internal use.
 
 # define some Dialog icons
 _ERROR = gtk.STOCK_DIALOG_ERROR
@@ -26,64 +26,36 @@ _last_message = None
 #
 def _set_details(dialog, details):
 
-    def expand(src, body, icon):
-        if (src.get_active()):
-            icon.set_from_stock(gtk.STOCK_ZOOM_OUT, gtk.ICON_SIZE_BUTTON)
-            body.show()
-        else:
-            icon.set_from_stock(gtk.STOCK_ZOOM_IN, gtk.ICON_SIZE_BUTTON)
-            body.hide()
-        dialog.resize(10, 10)
-        dialog.set_size_request(-1, -1)
-
-    #vbox1 = dialog.get_children[0]
-    vbox1 = dialog.get_vbox()
-
-    vbox2 = gtk.VBox()
-    vbox2.show()
-    vbox1.pack_start(vbox2)
-
-    align = gtk.Alignment(0.0, 0.0, 0.0, 0.0)
-    align.set_property("border-width", 6)
-    align.show()
-
-    expander = gtk.ToggleButton()
-    hbox = gtk.HBox()
-    hbox.show()
-    icon = gtk.Image()
-    icon.set_from_stock(gtk.STOCK_ZOOM_IN, gtk.ICON_SIZE_BUTTON)
-    icon.show()
-    hbox.add(icon)
-    lbl = gtk.Label(_("Details (%d lines)") % len(details.splitlines()))
-    lbl.show()
-    hbox.add(lbl)
-    expander.add(hbox)
+    vbox1 = gtk.VBox()
+    vbox2 = dialog.vbox
+    vbox2.pack_start(vbox1)
+    align1 = gtk.Alignment(0.0, 0.0, 0.0, 0.0)
+    align1.set_property("border-width", 6)
+    align1.show()
+    vbox2.pack_start(align1)
 
+    align2 = gtk.Alignment(0.0, 0.0, 0.0, 0.0)
+    align2.set_property("border-width", 6)
+    align2.show()
+    details = details.rstrip()
+    expander = gtk.expander_new_with_mnemonic(
+        _("_Details (%d lines)") % len(details.splitlines()))
     expander.show()
-    align.add(expander)
-    vbox2.pack_start(align)
-
-    align = gtk.Alignment(0.0, 0.0, 0.0, 0.0)
-    align.set_property("border-width", 6)
-    align.show()
-
     viewport = gtk.ScrolledWindow()
     viewport.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
-    lbl = gtk.Label(details)
-    lbl.set_property("selectable", True)
-    lbl.set_property("wrap", True)
-    lbl.set_selectable(True)
+    viewport.show()
+    lbl = LogView()
+    lbl.append(details)
     lbl.show()
-
     nil, height = lbl.get_size_request()
-    width, nil = vbox1.get_size_request()
-    viewport.set_size_request(width, min(height, 320))
+    width, nil = vbox2.get_size_request()
+    viewport.set_size_request(width, min(height, 480))
     viewport.add_with_viewport(lbl)
+    expander.add(viewport)
+    align2.add(expander)
 
-    align.add(viewport)
-    vbox2.pack_start(align)
-
-    expander.connect("toggled", expand, viewport, icon)
+    vbox1.show()
+    vbox1.pack_start(align2)
 
 
 #
@@ -108,8 +80,8 @@ def _queue_dialog(dialog):
 #
 def error(primary, secondary):
 
-    dialog = HIGDialog(_ERROR, primary, secondary,
-                       (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
+    dialog = HIGDialog((gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE),
+                       _ERROR, primary, secondary)
     gtk.threads_enter()
     dialog.run()
     gtk.threads_leave()
@@ -121,8 +93,8 @@ def error(primary, secondary):
 #
 def info(primary, secondary):
 
-    dialog = HIGDialog(_INFO, primary, secondary,
-                       (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
+    dialog = HIGDialog((gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE),
+                       _INFO, primary, secondary)
     _queue_dialog(dialog)
 
 
@@ -142,7 +114,7 @@ def question(icon, primary, secondary, *buttons):
         btns.append(response)
         response += 1
 
-    dialog = HIGDialog(_QUESTION, primary, secondary, tuple(btns))
+    dialog = HIGDialog(tuple(btns), _QUESTION, primary, secondary)
     dialog.connect("response", responder)
     dialog.show()
 
@@ -150,27 +122,20 @@ def question(icon, primary, secondary, *buttons):
 #
 # Displays a warning dialog.
 #
-def warning(primary, secondary, show_traceback = True, force = False):
+def warning(primary, secondary, details = "", force = False):
 
     global _last_message
 
     # don't show the same dialog twice in a sequence
     if (force): _last_message = ""
-    if (_last_message == (primary, secondary)): return
-    else: _last_message = (primary, secondary)
+    if (_last_message == (primary, secondary, details)): return
+    else: _last_message = (primary, secondary, details)
 
-    dialog = HIGDialog(_WARNING, primary, secondary,
-                       (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
+    dialog = HIGDialog((gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE),
+                       _WARNING, primary, secondary)
 
-    details = ""
 
-    if (show_traceback):
-        fd = StringIO()
-        traceback.print_exc(file = fd)
-        details = fd.getvalue()
-        fd.close()
-
-    if (details and not details.startswith("None")):
+    if (details):
         _set_details(dialog, details)
 
     _queue_dialog(dialog)
@@ -191,7 +156,8 @@ def fileselector(title, callback_ok, callback_cancel, *args):
 
     # do we have FileChooserDialog available?
     try:
-        fsel = gtk.FileChooserDialog(title, None, gtk.FILE_CHOOSER_ACTION_OPEN,
+        fsel = gtk.FileChooserDialog(title, None,
+                                     gtk.FILE_CHOOSER_ACTION_OPEN,
                                      (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                       gtk.STOCK_OPEN, gtk.RESPONSE_OK))
         fsel.connect("response", handler)
